@@ -12,10 +12,13 @@
 #include "src/api/api-inl.h"
 #include "src/base/compiler-specific.h"
 #include "src/base/sanitizer/asan.h"
+#include "src/common/allow-deprecated.h"
 #include "src/execution/vm-state-inl.h"
+#include "src/heap/base/stack.h"
 #include "src/heap/embedder-tracing.h"
 #include "src/heap/heap-inl.h"
 #include "src/heap/heap-write-barrier-inl.h"
+#include "src/heap/heap-write-barrier.h"
 #include "src/init/v8.h"
 #include "src/logging/counters.h"
 #include "src/objects/objects-inl.h"
@@ -30,10 +33,6 @@ namespace internal {
 
 namespace {
 
-// Specifies whether V8 expects the holder memory of a global handle to be live
-// or dead.
-enum class HandleHolder { kLive, kDead };
-
 constexpr size_t kBlockSize = 256;
 
 }  // namespace
@@ -384,7 +383,7 @@ namespace {
 
 void ExtractInternalFields(JSObject jsobject, void** embedder_fields, int len) {
   int field_count = jsobject.GetEmbedderFieldCount();
-  Isolate* isolate = GetIsolateForHeapSandbox(jsobject);
+  Isolate* isolate = GetIsolateForSandbox(jsobject);
   for (int i = 0; i < len; ++i) {
     if (field_count == i) break;
     void* pointer;
@@ -514,9 +513,11 @@ class GlobalHandles::Node final : public NodeBase<GlobalHandles::Node> {
       case v8::WeakCallbackType::kInternalFields:
         set_weakness_type(PHANTOM_WEAK_2_EMBEDDER_FIELDS);
         break;
+        START_ALLOW_USE_DEPRECATED()
       case v8::WeakCallbackType::kFinalizer:
         set_weakness_type(FINALIZER_WEAK);
         break;
+        END_ALLOW_USE_DEPRECATED()
     }
     set_parameter(parameter);
     weak_callback_ = phantom_callback;
@@ -569,8 +570,7 @@ class GlobalHandles::Node final : public NodeBase<GlobalHandles::Node> {
     set_state(NEAR_DEATH);
   }
 
-  void ResetPhantomHandle(HandleHolder handle_holder) {
-    DCHECK_EQ(HandleHolder::kLive, handle_holder);
+  void ResetPhantomHandle() {
     DCHECK_EQ(PHANTOM_WEAK_RESET_HANDLE, weakness_type());
     DCHECK_EQ(PENDING, state());
     DCHECK_NULL(weak_callback_);
@@ -648,7 +648,6 @@ class GlobalHandles::TracedNode final
   void MarkAsUsed() { set_state(NORMAL); }
   bool IsInUse() const { return state() != FREE; }
   bool IsRetainer() const { return state() == NORMAL; }
-  bool IsPhantomResetHandle() const { return callback_ == nullptr; }
 
   bool is_in_young_list() const { return IsInYoungList::decode(flags_); }
   void set_in_young_list(bool v) { flags_ = IsInYoungList::update(flags_, v); }
@@ -656,9 +655,6 @@ class GlobalHandles::TracedNode final
   bool is_root() const { return IsRoot::decode(flags_); }
   void set_root(bool v) { flags_ = IsRoot::update(flags_, v); }
 
-  bool has_destructor() const { return HasDestructor::decode(flags_); }
-  void set_has_destructor(bool v) { flags_ = HasDestructor::update(flags_, v); }
-
   bool markbit() const { return Markbit::decode(flags_); }
   void clear_markbit() { flags_ = Markbit::update(flags_, false); }
   void set_markbit() { flags_ = Markbit::update(flags_, true); }
@@ -666,40 +662,12 @@ class GlobalHandles::TracedNode final
   bool is_on_stack() const { return IsOnStack::decode(flags_); }
   void set_is_on_stack(bool v) { flags_ = IsOnStack::update(flags_, v); }
 
-  void SetFinalizationCallback(void* parameter,
-                               WeakCallbackInfo<void>::Callback callback) {
-    set_parameter(parameter);
-    callback_ = callback;
-  }
-  bool HasFinalizationCallback() const { return callback_ != nullptr; }
+  void clear_object() { object_ = kNullAddress; }
 
   void CopyObjectReference(const TracedNode& other) { object_ = other.object_; }
 
-  void CollectPhantomCallbackData(
-      std::vector<std::pair<TracedNode*, PendingPhantomCallback>>*
-          pending_phantom_callbacks) {
+  void ResetPhantomHandle() {
     DCHECK(IsInUse());
-    DCHECK_NOT_NULL(callback_);
-
-    void* embedder_fields[v8::kEmbedderFieldsInWeakCallback] = {nullptr,
-                                                                nullptr};
-    ExtractInternalFields(JSObject::cast(object()), embedder_fields,
-                          v8::kEmbedderFieldsInWeakCallback);
-
-    // Zap with something dangerous.
-    location().store(Object(0xCA11));
-
-    pending_phantom_callbacks->push_back(std::make_pair(
-        this, PendingPhantomCallback(callback_, parameter(), embedder_fields)));
-    set_state(NEAR_DEATH);
-  }
-
-  void ResetPhantomHandle(HandleHolder handle_holder) {
-    DCHECK(IsInUse());
-    if (handle_holder == HandleHolder::kLive) {
-      Address** handle = reinterpret_cast<Address**>(data_.parameter);
-      *handle = nullptr;
-    }
     NodeSpace<TracedNode>::Release(this);
     DCHECK(!IsInUse());
   }
@@ -710,27 +678,21 @@ class GlobalHandles::TracedNode final
   using NodeState = base::BitField8<State, 0, 2>;
   using IsInYoungList = NodeState::Next<bool, 1>;
   using IsRoot = IsInYoungList::Next<bool, 1>;
-  using HasDestructor = IsRoot::Next<bool, 1>;
-  using Markbit = HasDestructor::Next<bool, 1>;
+  using Markbit = IsRoot::Next<bool, 1>;
   using IsOnStack = Markbit::Next<bool, 1>;
 
   void ClearImplFields() {
     set_root(true);
     // Nodes are black allocated for simplicity.
     set_markbit();
-    callback_ = nullptr;
     set_is_on_stack(false);
-    set_has_destructor(false);
   }
 
   void CheckImplFieldsAreCleared() const {
     DCHECK(is_root());
     DCHECK(markbit());
-    DCHECK_NULL(callback_);
   }
 
-  WeakCallbackInfo<void>::Callback callback_;
-
   friend class NodeBase<GlobalHandles::TracedNode>;
 };
 
@@ -752,7 +714,7 @@ class GlobalHandles::OnStackTracedNodeSpace final {
 
   void SetStackStart(void* stack_start) {
     CHECK(on_stack_nodes_.empty());
-    stack_start_ = base::Stack::GetRealStackAddressForSlot(stack_start);
+    stack_.SetStackStart(base::Stack::GetRealStackAddressForSlot(stack_start));
   }
 
   V8_INLINE bool IsOnStack(uintptr_t slot) const;
@@ -788,20 +750,17 @@ class GlobalHandles::OnStackTracedNodeSpace final {
   std::map<uintptr_t, NodeEntry> on_stack_nodes_;
 #endif  // !V8_USE_ADDRESS_SANITIZER
 
-  uintptr_t stack_start_ = 0;
+  ::heap::base::Stack stack_;
   GlobalHandles* global_handles_ = nullptr;
   size_t acquire_count_ = 0;
 };
 
 bool GlobalHandles::OnStackTracedNodeSpace::IsOnStack(uintptr_t slot) const {
-#ifdef V8_USE_ADDRESS_SANITIZER
-  if (__asan_addr_is_in_fake_stack(__asan_get_current_fake_stack(),
-                                   reinterpret_cast<void*>(slot), nullptr,
-                                   nullptr)) {
-    return true;
-  }
-#endif  // V8_USE_ADDRESS_SANITIZER
-  return stack_start_ >= slot && slot > base::Stack::GetCurrentStackPosition();
+  // By the time this function is called, the stack start may not be set (i.e.
+  // SetStackStart() was not called). In that case, assume the slot is not on
+  // stack.
+  if (!stack_.stack_start()) return false;
+  return stack_.IsOnStack(reinterpret_cast<void*>(slot));
 }
 
 void GlobalHandles::OnStackTracedNodeSpace::NotifyEmptyEmbedderStack() {
@@ -868,20 +827,32 @@ GlobalHandles::TracedNode* GlobalHandles::OnStackTracedNodeSpace::Acquire(
 
 void GlobalHandles::OnStackTracedNodeSpace::CleanupBelowCurrentStackPosition() {
   if (on_stack_nodes_.empty()) return;
-  const auto it =
-      on_stack_nodes_.upper_bound(base::Stack::GetCurrentStackPosition());
+  const uintptr_t stack_ptr = reinterpret_cast<uintptr_t>(
+      ::heap::base::Stack::GetCurrentStackPointerForLocalVariables());
+  const auto it = on_stack_nodes_.upper_bound(stack_ptr);
   on_stack_nodes_.erase(on_stack_nodes_.begin(), it);
 }
 
+// static
+void GlobalHandles::EnableMarkingBarrier(Isolate* isolate) {
+  auto* global_handles = isolate->global_handles();
+  DCHECK(!global_handles->is_marking_);
+  global_handles->is_marking_ = true;
+}
+
+// static
+void GlobalHandles::DisableMarkingBarrier(Isolate* isolate) {
+  auto* global_handles = isolate->global_handles();
+  DCHECK(global_handles->is_marking_);
+  global_handles->is_marking_ = false;
+}
+
 // static
 void GlobalHandles::TracedNode::Verify(GlobalHandles* global_handles,
                                        const Address* const* slot) {
 #ifdef DEBUG
   const TracedNode* node = FromLocation(*slot);
   DCHECK(node->IsInUse());
-  DCHECK_IMPLIES(!node->has_destructor(), nullptr == node->parameter());
-  DCHECK_IMPLIES(node->has_destructor() && !node->HasFinalizationCallback(),
-                 node->parameter());
   bool slot_on_stack = global_handles->on_stack_nodes_->IsOnStack(
       reinterpret_cast<uintptr_t>(slot));
   DCHECK_EQ(slot_on_stack, node->is_on_stack());
@@ -949,14 +920,14 @@ Handle<Object> GlobalHandles::Create(Address value) {
 }
 
 Handle<Object> GlobalHandles::CreateTraced(Object value, Address* slot,
-                                           bool has_destructor) {
+                                           GlobalHandleStoreMode store_mode) {
   return CreateTraced(
-      value, slot, has_destructor,
+      value, slot, store_mode,
       on_stack_nodes_->IsOnStack(reinterpret_cast<uintptr_t>(slot)));
 }
 
 Handle<Object> GlobalHandles::CreateTraced(Object value, Address* slot,
-                                           bool has_destructor,
+                                           GlobalHandleStoreMode store_mode,
                                            bool is_on_stack) {
   GlobalHandles::TracedNode* result;
   if (is_on_stack) {
@@ -967,15 +938,17 @@ Handle<Object> GlobalHandles::CreateTraced(Object value, Address* slot,
       traced_young_nodes_.push_back(result);
       result->set_in_young_list(true);
     }
+    if (store_mode != GlobalHandleStoreMode::kInitializingStore) {
+      WriteBarrier::MarkingFromGlobalHandle(value);
+    }
   }
-  result->set_has_destructor(has_destructor);
-  result->set_parameter(has_destructor ? slot : nullptr);
+  result->set_parameter(nullptr);
   return result->handle();
 }
 
 Handle<Object> GlobalHandles::CreateTraced(Address value, Address* slot,
-                                           bool has_destructor) {
-  return CreateTraced(Object(value), slot, has_destructor);
+                                           GlobalHandleStoreMode store_mode) {
+  return CreateTraced(Object(value), slot, store_mode);
 }
 
 Handle<Object> GlobalHandles::CopyGlobal(Address* location) {
@@ -998,21 +971,16 @@ void SetSlotThreadSafe(Address** slot, Address* val) {
 }  // namespace
 
 // static
-void GlobalHandles::CopyTracedGlobal(const Address* const* from, Address** to) {
+void GlobalHandles::CopyTracedReference(const Address* const* from,
+                                        Address** to) {
   DCHECK_NOT_NULL(*from);
   DCHECK_NULL(*to);
   const TracedNode* node = TracedNode::FromLocation(*from);
-  // Copying a traced handle with finalization callback is prohibited because
-  // the callback may require knowing about multiple copies of the traced
-  // handle.
-  CHECK_WITH_MSG(!node->HasFinalizationCallback(),
-                 "Copying of references is not supported when "
-                 "SetFinalizationCallback is set.");
-
   GlobalHandles* global_handles =
       GlobalHandles::From(const_cast<TracedNode*>(node));
   Handle<Object> o = global_handles->CreateTraced(
-      node->object(), reinterpret_cast<Address*>(to), node->has_destructor());
+      node->object(), reinterpret_cast<Address*>(to),
+      GlobalHandleStoreMode::kAssigningStore);
   SetSlotThreadSafe(to, o.location());
   TracedNode::Verify(global_handles, from);
   TracedNode::Verify(global_handles, to);
@@ -1037,10 +1005,10 @@ void GlobalHandles::MoveGlobal(Address** from, Address** to) {
   //   those the callers need to ensure consistency.
 }
 
-void GlobalHandles::MoveTracedGlobal(Address** from, Address** to) {
+void GlobalHandles::MoveTracedReference(Address** from, Address** to) {
   // Fast path for moving from an empty reference.
   if (!*from) {
-    DestroyTraced(*to);
+    DestroyTracedReference(*to);
     SetSlotThreadSafe(to, nullptr);
     return;
   }
@@ -1064,17 +1032,6 @@ void GlobalHandles::MoveTracedGlobal(Address** from, Address** to) {
     to_on_stack = to_node->is_on_stack();
   }
 
-  // Moving a traced handle with finalization callback is prohibited because
-  // the callback may require knowing about multiple copies of the traced
-  // handle.
-  CHECK_WITH_MSG(!from_node->HasFinalizationCallback(),
-                 "Moving of references is not supported when "
-                 "SetFinalizationCallback is set.");
-  // Types in v8.h ensure that we only copy/move handles that have the same
-  // destructor behavior.
-  DCHECK_IMPLIES(to_node,
-                 to_node->has_destructor() == from_node->has_destructor());
-
   // Moving.
   if (from_on_stack || to_on_stack) {
     // Move involving a stack slot.
@@ -1082,7 +1039,7 @@ void GlobalHandles::MoveTracedGlobal(Address** from, Address** to) {
       DCHECK(global_handles);
       Handle<Object> o = global_handles->CreateTraced(
           from_node->object(), reinterpret_cast<Address*>(to),
-          from_node->has_destructor(), to_on_stack);
+          GlobalHandleStoreMode::kAssigningStore, to_on_stack);
       SetSlotThreadSafe(to, o.location());
       to_node = TracedNode::FromLocation(*to);
       DCHECK(to_node->markbit());
@@ -1095,21 +1052,21 @@ void GlobalHandles::MoveTracedGlobal(Address** from, Address** to) {
         global_handles->traced_young_nodes_.push_back(to_node);
         to_node->set_in_young_list(true);
       }
+      if (!to_on_stack) {
+        WriteBarrier::MarkingFromGlobalHandle(to_node->object());
+      }
     }
-    DestroyTraced(*from);
+    DestroyTracedReference(*from);
     SetSlotThreadSafe(from, nullptr);
   } else {
     // Pure heap move.
-    DestroyTraced(*to);
+    DestroyTracedReference(*to);
     SetSlotThreadSafe(to, *from);
     to_node = from_node;
     DCHECK_NOT_NULL(*from);
     DCHECK_NOT_NULL(*to);
     DCHECK_EQ(*from, *to);
-    // Fixup back reference for destructor.
-    if (to_node->has_destructor()) {
-      to_node->set_parameter(to);
-    }
+    WriteBarrier::MarkingFromGlobalHandle(to_node->object());
     SetSlotThreadSafe(from, nullptr);
   }
   TracedNode::Verify(global_handles, to);
@@ -1134,22 +1091,42 @@ void GlobalHandles::Destroy(Address* location) {
   }
 }
 
-void GlobalHandles::DestroyTraced(Address* location) {
+// static
+void GlobalHandles::DestroyTracedReference(Address* location) {
   if (location != nullptr) {
     TracedNode* node = TracedNode::FromLocation(location);
     if (node->is_on_stack()) {
       node->Release(nullptr);
-    } else {
-      NodeSpace<TracedNode>::Release(node);
+      return;
     }
-  }
-}
+    DCHECK(!node->is_on_stack());
 
-void GlobalHandles::SetFinalizationCallbackForTraced(
-    Address* location, void* parameter,
-    WeakCallbackInfo<void>::Callback callback) {
-  TracedNode::FromLocation(location)->SetFinalizationCallback(parameter,
-                                                              callback);
+    auto* global_handles = GlobalHandles::From(node);
+    // When marking is off the handle may be freed immediately. Note that this
+    // includes also the case when invoking the first pass callbacks during the
+    // atomic pause which requires releasing a node fully.
+    if (!global_handles->is_marking_) {
+      NodeSpace<TracedNode>::Release(node);
+      return;
+    }
+
+    // Incremental marking is on. This also covers the scavenge case which
+    // prohibits eagerly reclaiming nodes when marking is on during a scavenge.
+    //
+    // On-heap traced nodes are released in the atomic pause in
+    // `IterateWeakRootsForPhantomHandles()` when they are discovered as not
+    // marked.
+    //
+    // Eagerly clear out the object here to avoid needlessly marking it from
+    // this point on. Also clear out callback and backreference for the version
+    // with callbacks to avoid calling into possibly dead memory later.
+    //
+    // In the case this happens during incremental marking, the node may
+    // still be spuriously marked as live and is then only reclaimed on the
+    // next cycle.
+    node->clear_object();
+    node->set_parameter(nullptr);
+  }
 }
 
 using GenericCallback = v8::WeakCallbackInfo<void>::Callback;
@@ -1198,7 +1175,7 @@ void GlobalHandles::IterateWeakRootsForPhantomHandles(
         should_reset_handle(isolate()->heap(), node->location())) {
       if (node->IsPhantomResetHandle()) {
         node->MarkPending();
-        node->ResetPhantomHandle(HandleHolder::kLive);
+        node->ResetPhantomHandle();
         ++number_of_phantom_handle_resets_;
       } else if (node->IsPhantomCallback()) {
         node->MarkPending();
@@ -1209,31 +1186,20 @@ void GlobalHandles::IterateWeakRootsForPhantomHandles(
   for (TracedNode* node : *traced_nodes_) {
     if (!node->IsInUse()) continue;
     // Detect unreachable nodes first.
-    if (!node->markbit() && node->IsPhantomResetHandle() &&
-        !node->has_destructor()) {
-      // The handle is unreachable and does not have a callback and a
-      // destructor associated with it. We can clear it even if the target V8
-      // object is alive. Note that the desctructor and the callback may
-      // access the handle, that is why we avoid clearing it.
-      node->ResetPhantomHandle(HandleHolder::kDead);
+    if (!node->markbit()) {
+      // The handle itself is unreachable. We can clear it even if the target V8
+      // object is alive.
+      node->ResetPhantomHandle();
       ++number_of_phantom_handle_resets_;
       continue;
-    } else if (node->markbit()) {
-      // Clear the markbit for the next GC.
-      node->clear_markbit();
     }
+    // Clear the markbit for the next GC.
+    node->clear_markbit();
     DCHECK(node->IsInUse());
     // Detect nodes with unreachable target objects.
     if (should_reset_handle(isolate()->heap(), node->location())) {
-      // If the node allows eager resetting, then reset it here. Otherwise,
-      // collect its callback that will reset it.
-      if (node->IsPhantomResetHandle()) {
-        node->ResetPhantomHandle(node->has_destructor() ? HandleHolder::kLive
-                                                        : HandleHolder::kDead);
-        ++number_of_phantom_handle_resets_;
-      } else {
-        node->CollectPhantomCallbackData(&traced_pending_phantom_callbacks_);
-      }
+      node->ResetPhantomHandle();
+      ++number_of_phantom_handle_resets_;
     }
   }
 }
@@ -1264,13 +1230,8 @@ void GlobalHandles::IdentifyWeakUnmodifiedObjects(
       DCHECK(node->is_root());
       if (is_unmodified(node->location())) {
         v8::Value* value = ToApi<v8::Value>(node->handle());
-        if (node->has_destructor()) {
-          node->set_root(handler->IsRoot(
-              *reinterpret_cast<v8::TracedGlobal<v8::Value>*>(&value)));
-        } else {
-          node->set_root(handler->IsRoot(
-              *reinterpret_cast<v8::TracedReference<v8::Value>*>(&value)));
-        }
+        node->set_root(handler->IsRoot(
+            *reinterpret_cast<v8::TracedReference<v8::Value>*>(&value)));
       }
     }
   }
@@ -1324,7 +1285,7 @@ void GlobalHandles::IterateYoungWeakObjectsForPhantomHandles(
         DCHECK(node->IsPhantomResetHandle() || node->IsPhantomCallback());
         if (node->IsPhantomResetHandle()) {
           node->MarkPending();
-          node->ResetPhantomHandle(HandleHolder::kLive);
+          node->ResetPhantomHandle();
           ++number_of_phantom_handle_resets_;
         } else if (node->IsPhantomCallback()) {
           node->MarkPending();
@@ -1349,23 +1310,13 @@ void GlobalHandles::IterateYoungWeakObjectsForPhantomHandles(
     DCHECK_IMPLIES(node->is_root(),
                    !should_reset_handle(isolate_->heap(), node->location()));
     if (should_reset_handle(isolate_->heap(), node->location())) {
-      if (node->IsPhantomResetHandle()) {
-        if (node->has_destructor()) {
-          // For handles with destructor it is guaranteed that the embedder
-          // memory is still alive as the destructor would have otherwise
-          // removed the memory.
-          node->ResetPhantomHandle(HandleHolder::kLive);
-        } else {
-          v8::Value* value = ToApi<v8::Value>(node->handle());
-          handler->ResetRoot(
-              *reinterpret_cast<v8::TracedReference<v8::Value>*>(&value));
-          DCHECK(!node->IsInUse());
-        }
-
-        ++number_of_phantom_handle_resets_;
-      } else {
-        node->CollectPhantomCallbackData(&traced_pending_phantom_callbacks_);
-      }
+      v8::Value* value = ToApi<v8::Value>(node->handle());
+      handler->ResetRoot(
+          *reinterpret_cast<v8::TracedReference<v8::Value>*>(&value));
+      // We cannot check whether a node is in use here as the reset behavior
+      // depends on whether incremental marking is running when reclaiming
+      // young objects.
+      ++number_of_phantom_handle_resets_;
     } else {
       if (!node->is_root()) {
         node->set_root(true);
@@ -1649,13 +1600,8 @@ void GlobalHandles::IterateTracedNodes(
   for (TracedNode* node : *traced_nodes_) {
     if (node->IsInUse()) {
       v8::Value* value = ToApi<v8::Value>(node->handle());
-      if (node->has_destructor()) {
-        visitor->VisitTracedGlobalHandle(
-            *reinterpret_cast<v8::TracedGlobal<v8::Value>*>(&value));
-      } else {
-        visitor->VisitTracedReference(
-            *reinterpret_cast<v8::TracedReference<v8::Value>*>(&value));
-      }
+      visitor->VisitTracedReference(
+          *reinterpret_cast<v8::TracedReference<v8::Value>*>(&value));
     }
   }
 }
