@@ -16,7 +16,23 @@
 namespace v8 {
 namespace internal {
 
-enum class StoreMode { kOrdinary, kInLiteral };
+enum class StoreMode {
+  // TODO(v8:12548): rename to kSet and kDefineKeyedOwnInLiteral
+  kOrdinary,
+  kInLiteral,
+
+  // kDefineNamedOwn performs an ordinary property store without traversing the
+  // prototype chain. In the case of private fields, it will throw if the
+  // field does not already exist.
+  // kDefineKeyedOwn is similar to kDefineNamedOwn, but for private class
+  // fields, it will throw if the field does already exist.
+  kDefineNamedOwn,
+  kDefineKeyedOwn
+};
+
+// With private symbols, 'define' semantics will throw if the field already
+// exists, while 'update' semantics will throw if the field does not exist.
+enum class PrivateNameSemantics { kUpdate, kDefine };
 
 class KeyedStoreGenericAssembler : public AccessorAssembler {
  public:
@@ -68,16 +84,18 @@ class KeyedStoreGenericAssembler : public AccessorAssembler {
   // kind.
   void EmitGenericPropertyStore(TNode<JSReceiver> receiver,
                                 TNode<Map> receiver_map,
+                                TNode<Uint16T> instance_type,
                                 const StoreICParameters* p,
                                 ExitPoint* exit_point, Label* slow,
                                 Maybe<LanguageMode> maybe_language_mode);
 
   void EmitGenericPropertyStore(TNode<JSReceiver> receiver,
                                 TNode<Map> receiver_map,
+                                TNode<Uint16T> instance_type,
                                 const StoreICParameters* p, Label* slow) {
     ExitPoint direct_exit(this);
-    EmitGenericPropertyStore(receiver, receiver_map, p, &direct_exit, slow,
-                             Nothing<LanguageMode>());
+    EmitGenericPropertyStore(receiver, receiver_map, instance_type, p,
+                             &direct_exit, slow, Nothing<LanguageMode>());
   }
 
   void BranchIfPrototypesMayHaveReadOnlyElements(
@@ -131,14 +149,21 @@ class KeyedStoreGenericAssembler : public AccessorAssembler {
 
   bool IsKeyedStore() const { return mode_ == StoreMode::kOrdinary; }
   bool IsStoreInLiteral() const { return mode_ == StoreMode::kInLiteral; }
+  bool IsDefineNamedOwn() const { return mode_ == StoreMode::kDefineNamedOwn; }
+  bool IsDefineKeyedOwn() const { return mode_ == StoreMode::kDefineKeyedOwn; }
 
   bool ShouldCheckPrototype() const { return IsKeyedStore(); }
   bool ShouldReconfigureExisting() const { return IsStoreInLiteral(); }
   bool ShouldCallSetter() const { return IsKeyedStore(); }
   bool ShouldCheckPrototypeValidity() const {
     // We don't do this for "in-literal" stores, because it is impossible for
-    // the target object to be a "prototype"
-    return !IsStoreInLiteral();
+    // the target object to be a "prototype".
+    // We don't need the prototype validity check for "own" stores, because
+    // we don't care about the prototype chain.
+    // Thus, we need the prototype check only for ordinary stores.
+    DCHECK_IMPLIES(!IsKeyedStore(), IsStoreInLiteral() || IsDefineNamedOwn() ||
+                                        IsDefineKeyedOwn());
+    return IsKeyedStore();
   }
 };
 
@@ -147,11 +172,25 @@ void KeyedStoreGenericGenerator::Generate(compiler::CodeAssemblerState* state) {
   assembler.KeyedStoreGeneric();
 }
 
+void DefineKeyedOwnGenericGenerator::Generate(
+    compiler::CodeAssemblerState* state) {
+  KeyedStoreGenericAssembler assembler(state, StoreMode::kDefineKeyedOwn);
+  assembler.KeyedStoreGeneric();
+}
+
 void StoreICNoFeedbackGenerator::Generate(compiler::CodeAssemblerState* state) {
   KeyedStoreGenericAssembler assembler(state, StoreMode::kOrdinary);
   assembler.StoreIC_NoFeedback();
 }
 
+void DefineNamedOwnICNoFeedbackGenerator::Generate(
+    compiler::CodeAssemblerState* state) {
+  // TODO(v8:12548): it's a hack to reuse KeyedStoreGenericAssembler for
+  // DefineNamedOwnIC, we should separate it out.
+  KeyedStoreGenericAssembler assembler(state, StoreMode::kDefineNamedOwn);
+  assembler.StoreIC_NoFeedback();
+}
+
 void KeyedStoreGenericGenerator::SetProperty(
     compiler::CodeAssemblerState* state, TNode<Context> context,
     TNode<JSReceiver> receiver, TNode<BoolT> is_simple_receiver,
@@ -740,7 +779,7 @@ TNode<Map> KeyedStoreGenericAssembler::FindCandidateStoreICTransitionMapHandler(
       // transition array is expected to be the first among the transitions
       // with the same name.
       // See TransitionArray::CompareDetails() for details.
-      STATIC_ASSERT(kData == 0);
+      STATIC_ASSERT(static_cast<int>(PropertyKind::kData) == 0);
       STATIC_ASSERT(NONE == 0);
       const int kKeyToTargetOffset = (TransitionArray::kEntryTargetIndex -
                                       TransitionArray::kEntryKeyIndex) *
@@ -758,7 +797,8 @@ TNode<Map> KeyedStoreGenericAssembler::FindCandidateStoreICTransitionMapHandler(
 
 void KeyedStoreGenericAssembler::EmitGenericPropertyStore(
     TNode<JSReceiver> receiver, TNode<Map> receiver_map,
-    const StoreICParameters* p, ExitPoint* exit_point, Label* slow,
+    TNode<Uint16T> instance_type, const StoreICParameters* p,
+    ExitPoint* exit_point, Label* slow,
     Maybe<LanguageMode> maybe_language_mode) {
   CSA_DCHECK(this, IsSimpleObjectMap(receiver_map));
   // TODO(rmcilroy) Type as Struct once we use a trimmed down
@@ -783,6 +823,10 @@ void KeyedStoreGenericAssembler::EmitGenericPropertyStore(
 
     BIND(&descriptor_found);
     {
+      if (IsDefineKeyedOwn()) {
+        // Take slow path to throw if a private name already exists.
+        GotoIf(IsPrivateSymbol(name), slow);
+      }
       TNode<IntPtrT> name_index = var_name_index.value();
       TNode<Uint32T> details = LoadDetailsByKeyIndex(descriptors, name_index);
       Label data_property(this);
@@ -803,11 +847,22 @@ void KeyedStoreGenericAssembler::EmitGenericPropertyStore(
 
       BIND(&data_property);
       {
+        Label shared(this);
+        GotoIf(IsJSSharedStructInstanceType(instance_type), &shared);
+
         CheckForAssociatedProtector(name, slow);
         OverwriteExistingFastDataProperty(receiver, receiver_map, descriptors,
                                           name_index, details, p->value(), slow,
                                           false);
         exit_point->Return(p->value());
+
+        BIND(&shared);
+        {
+          StoreJSSharedStructField(p->context(), receiver, receiver_map,
+                                   descriptors, name_index, details,
+                                   p->value());
+          exit_point->Return(p->value());
+        }
       }
     }
     BIND(&lookup_transition);
@@ -841,6 +896,10 @@ void KeyedStoreGenericAssembler::EmitGenericPropertyStore(
     BIND(&dictionary_found);
     {
       Label check_const(this), overwrite(this), done(this);
+      if (IsDefineKeyedOwn()) {
+        // Take slow path to throw if a private name already exists.
+        GotoIf(IsPrivateSymbol(name), slow);
+      }
       TNode<Uint32T> details =
           LoadDetailsByKeyIndex(properties, var_name_index.value());
       JumpIfDataProperty(details, &check_const,
@@ -966,20 +1025,25 @@ void KeyedStoreGenericAssembler::EmitGenericPropertyStore(
   if (!ShouldReconfigureExisting()) {
     BIND(&readonly);
     {
-      LanguageMode language_mode;
-      if (maybe_language_mode.To(&language_mode)) {
-        if (language_mode == LanguageMode::kStrict) {
-          TNode<String> type = Typeof(p->receiver());
-          ThrowTypeError(p->context(), MessageTemplate::kStrictReadOnlyProperty,
-                         name, type, p->receiver());
+      if (IsDefineKeyedOwn() || IsDefineNamedOwn()) {
+        Goto(slow);
+      } else {
+        LanguageMode language_mode;
+        if (maybe_language_mode.To(&language_mode)) {
+          if (language_mode == LanguageMode::kStrict) {
+            TNode<String> type = Typeof(p->receiver());
+            ThrowTypeError(p->context(),
+                           MessageTemplate::kStrictReadOnlyProperty, name, type,
+                           p->receiver());
+          } else {
+            exit_point->Return(p->value());
+          }
         } else {
+          CallRuntime(Runtime::kThrowTypeErrorIfStrict, p->context(),
+                      SmiConstant(MessageTemplate::kStrictReadOnlyProperty),
+                      name, Typeof(p->receiver()), p->receiver());
           exit_point->Return(p->value());
         }
-      } else {
-        CallRuntime(Runtime::kThrowTypeErrorIfStrict, p->context(),
-                    SmiConstant(MessageTemplate::kStrictReadOnlyProperty), name,
-                    Typeof(p->receiver()), p->receiver());
-        exit_point->Return(p->value());
       }
     }
   }
@@ -1016,10 +1080,10 @@ void KeyedStoreGenericAssembler::KeyedStoreGeneric(
   {
     Comment("key is unique name");
     StoreICParameters p(context, receiver, var_unique.value(), value, {},
-                        UndefinedConstant());
+                        UndefinedConstant(), StoreICMode::kDefault);
     ExitPoint direct_exit(this);
-    EmitGenericPropertyStore(CAST(receiver), receiver_map, &p, &direct_exit,
-                             &slow, language_mode);
+    EmitGenericPropertyStore(CAST(receiver), receiver_map, instance_type, &p,
+                             &direct_exit, &slow, language_mode);
   }
 
   BIND(&not_internalized);
@@ -1034,14 +1098,19 @@ void KeyedStoreGenericAssembler::KeyedStoreGeneric(
 
   BIND(&slow);
   {
-    if (IsKeyedStore()) {
+    if (IsKeyedStore() || IsDefineNamedOwn()) {
+      // The DefineNamedOwnIC hacky reuse should never reach here.
+      CSA_DCHECK(this, BoolConstant(!IsDefineNamedOwn()));
       Comment("KeyedStoreGeneric_slow");
       TailCallRuntime(Runtime::kSetKeyedProperty, context, receiver, key,
                       value);
+    } else if (IsDefineKeyedOwn()) {
+      TailCallRuntime(Runtime::kDefineObjectOwnProperty, context, receiver, key,
+                      value);
     } else {
       DCHECK(IsStoreInLiteral());
-      TailCallRuntime(Runtime::kStoreDataPropertyInLiteral, context, receiver,
-                      key, value);
+      TailCallRuntime(Runtime::kDefineKeyedOwnPropertyInLiteral_Simple, context,
+                      receiver, key, value);
     }
   }
 }
@@ -1087,15 +1156,20 @@ void KeyedStoreGenericAssembler::StoreIC_NoFeedback() {
     GotoIf(IsSpecialReceiverInstanceType(instance_type), &miss);
     {
       StoreICParameters p(context, receiver, name, value, slot,
-                          UndefinedConstant());
-      EmitGenericPropertyStore(CAST(receiver), receiver_map, &p, &miss);
+                          UndefinedConstant(),
+                          IsDefineNamedOwn() ? StoreICMode::kDefineNamedOwn
+                                             : StoreICMode::kDefault);
+      EmitGenericPropertyStore(CAST(receiver), receiver_map, instance_type, &p,
+                               &miss);
     }
   }
 
   BIND(&miss);
   {
-    TailCallRuntime(Runtime::kStoreIC_Miss, context, value, slot,
-                    UndefinedConstant(), receiver_maybe_smi, name);
+    auto runtime = IsDefineNamedOwn() ? Runtime::kDefineNamedOwnIC_Miss
+                                      : Runtime::kStoreIC_Miss;
+    TailCallRuntime(runtime, context, value, slot, UndefinedConstant(),
+                    receiver_maybe_smi, name);
   }
 }
 
@@ -1106,7 +1180,7 @@ void KeyedStoreGenericAssembler::SetProperty(TNode<Context> context,
                                              TNode<Object> value,
                                              LanguageMode language_mode) {
   StoreICParameters p(context, receiver, unique_name, value, {},
-                      UndefinedConstant());
+                      UndefinedConstant(), StoreICMode::kDefault);
 
   Label done(this), slow(this, Label::kDeferred);
   ExitPoint exit_point(this, [&](TNode<Object> result) { Goto(&done); });
@@ -1115,14 +1189,16 @@ void KeyedStoreGenericAssembler::SetProperty(TNode<Context> context,
                                IsSimpleObjectMap(LoadMap(receiver))));
   GotoIfNot(is_simple_receiver, &slow);
 
-  EmitGenericPropertyStore(receiver, LoadMap(receiver), &p, &exit_point, &slow,
+  TNode<Map> map = LoadMap(receiver);
+  TNode<Uint16T> instance_type = LoadMapInstanceType(map);
+  EmitGenericPropertyStore(receiver, map, instance_type, &p, &exit_point, &slow,
                            Just(language_mode));
 
   BIND(&slow);
   {
     if (IsStoreInLiteral()) {
-      CallRuntime(Runtime::kStoreDataPropertyInLiteral, context, receiver,
-                  unique_name, value);
+      CallRuntime(Runtime::kDefineKeyedOwnPropertyInLiteral_Simple, context,
+                  receiver, unique_name, value);
     } else {
       CallRuntime(Runtime::kSetKeyedProperty, context, receiver, unique_name,
                   value);
