@@ -28,6 +28,10 @@ class BytecodeArray;
 class CodeDataContainer;
 class CodeDesc;
 
+class LocalFactory;
+template <typename Impl>
+class FactoryBase;
+
 namespace interpreter {
 class Register;
 }  // namespace interpreter
@@ -49,11 +53,36 @@ class CodeDataContainer : public HeapObject {
   // is deterministic.
   inline void clear_padding();
 
+  //
+  // A collection of getters and predicates that are used by respective methods
+  // on Code object. They are defined here mostly because they operate on the
+  // writable state of the respective Code object.
+  //
+
+  inline bool can_have_weak_objects() const;
+  inline void set_can_have_weak_objects(bool value);
+
+  inline bool marked_for_deoptimization() const;
+  inline void set_marked_for_deoptimization(bool flag);
+
   // Back-reference to the Code object.
   // Available only when V8_EXTERNAL_CODE_SPACE is defined.
   DECL_GETTER(code, Code)
   DECL_RELAXED_GETTER(code, Code)
 
+  // When V8_EXTERNAL_CODE_SPACE is enabled, Code objects are allocated in
+  // a separate pointer compression cage instead of the cage where all the
+  // other objects are allocated.
+  // This field contains code cage base value which is used for decompressing
+  // the reference to respective Code. Basically, |code_cage_base| and |code|
+  // fields together form a full pointer. The reason why they are split is that
+  // the code field must also support atomic access and the word alignment of
+  // the full value is not guaranteed.
+  inline PtrComprCageBase code_cage_base() const;
+  inline void set_code_cage_base(Address code_cage_base);
+  inline PtrComprCageBase code_cage_base(RelaxedLoadTag) const;
+  inline void set_code_cage_base(Address code_cage_base, RelaxedStoreTag);
+
   // Cached value of code().InstructionStart().
   // Available only when V8_EXTERNAL_CODE_SPACE is defined.
   DECL_GETTER(code_entry_point, Address)
@@ -67,9 +96,62 @@ class CodeDataContainer : public HeapObject {
 
   inline void AllocateExternalPointerEntries(Isolate* isolate);
 
+  // Initializes internal flags field which stores cached values of some
+  // properties of the respective Code object.
+  // Available only when V8_EXTERNAL_CODE_SPACE is enabled.
+  inline void initialize_flags(CodeKind kind, Builtin builtin_id);
+
   // Alias for code_entry_point to make it API compatible with Code.
   inline Address InstructionStart() const;
 
+  // Alias for code_entry_point to make it API compatible with Code.
+  inline Address raw_instruction_start();
+
+  // Alias for code_entry_point to make it API compatible with Code.
+  inline Address entry() const;
+
+#ifdef V8_EXTERNAL_CODE_SPACE
+  //
+  // A collection of getters and predicates that forward queries to associated
+  // Code object.
+  //
+
+  inline CodeKind kind() const;
+  inline Builtin builtin_id() const;
+  inline bool is_builtin() const;
+
+  inline bool is_optimized_code() const;
+  inline bool is_wasm_code() const;
+
+  // Testers for interpreter builtins.
+  inline bool is_interpreter_trampoline_builtin() const;
+
+  // Testers for baseline builtins.
+  inline bool is_baseline_trampoline_builtin() const;
+  inline bool is_baseline_leave_frame_builtin() const;
+
+  // Tells whether the code checks the optimization marker in the function's
+  // feedback vector.
+  inline bool checks_optimization_marker() const;
+
+  // Tells whether the outgoing parameters of this code are tagged pointers.
+  inline bool has_tagged_outgoing_params() const;
+
+  // [is_turbofanned]: Tells whether the code object was generated by the
+  // TurboFan optimizing compiler.
+  inline bool is_turbofanned() const;
+
+  // [is_off_heap_trampoline]: For kind BUILTIN tells whether
+  // this is a trampoline to an off-heap builtin.
+  inline bool is_off_heap_trampoline() const;
+
+  DECL_GETTER(deoptimization_data, FixedArray)
+  DECL_GETTER(bytecode_or_interpreter_data, HeapObject)
+  DECL_GETTER(source_position_table, ByteArray)
+  DECL_GETTER(bytecode_offset_table, ByteArray)
+
+#endif  // V8_EXTERNAL_CODE_SPACE
+
   DECL_CAST(CodeDataContainer)
 
   // Dispatched behavior.
@@ -77,21 +159,25 @@ class CodeDataContainer : public HeapObject {
   DECL_VERIFIER(CodeDataContainer)
 
 // Layout description.
-#define CODE_DATA_FIELDS(V)                                     \
-  /* Strong pointer fields. */                                  \
-  V(kPointerFieldsStrongEndOffset, 0)                           \
-  /* Weak pointer fields. */                                    \
-  V(kNextCodeLinkOffset, kTaggedSize)                           \
-  V(kPointerFieldsWeakEndOffset, 0)                             \
-  /* Strong Code pointer fields. */                             \
-  V(kCodeOffset, V8_EXTERNAL_CODE_SPACE_BOOL ? kTaggedSize : 0) \
-  V(kCodePointerFieldsStrongEndOffset, 0)                       \
-  /* Raw data fields. */                                        \
-  V(kCodeEntryPointOffset,                                      \
-    V8_EXTERNAL_CODE_SPACE_BOOL ? kExternalPointerSize : 0)     \
-  V(kKindSpecificFlagsOffset, kInt32Size)                       \
-  V(kUnalignedSize, OBJECT_POINTER_PADDING(kUnalignedSize))     \
-  /* Total size. */                                             \
+#define CODE_DATA_FIELDS(V)                                         \
+  /* Strong pointer fields. */                                      \
+  V(kPointerFieldsStrongEndOffset, 0)                               \
+  /* Weak pointer fields. */                                        \
+  V(kNextCodeLinkOffset, kTaggedSize)                               \
+  V(kPointerFieldsWeakEndOffset, 0)                                 \
+  /* Strong Code pointer fields. */                                 \
+  V(kCodeOffset, V8_EXTERNAL_CODE_SPACE_BOOL ? kTaggedSize : 0)     \
+  V(kCodePointerFieldsStrongEndOffset, 0)                           \
+  /* Raw data fields. */                                            \
+  V(kCodeCageBaseUpper32BitsOffset,                                 \
+    V8_EXTERNAL_CODE_SPACE_BOOL ? kTaggedSize : 0)                  \
+  V(kCodeEntryPointOffset,                                          \
+    V8_EXTERNAL_CODE_SPACE_BOOL ? kExternalPointerSize : 0)         \
+  V(kFlagsOffset, V8_EXTERNAL_CODE_SPACE_BOOL ? kUInt16Size : 0)    \
+  V(kBuiltinIdOffset, V8_EXTERNAL_CODE_SPACE_BOOL ? kInt16Size : 0) \
+  V(kKindSpecificFlagsOffset, kInt32Size)                           \
+  V(kUnalignedSize, OBJECT_POINTER_PADDING(kUnalignedSize))         \
+  /* Total size. */                                                 \
   V(kSize, 0)
 
   DEFINE_FIELD_OFFSET_CONSTANTS(HeapObject::kHeaderSize, CODE_DATA_FIELDS)
@@ -99,12 +185,30 @@ class CodeDataContainer : public HeapObject {
 
   class BodyDescriptor;
 
+  // Flags layout.
+#define FLAGS_BIT_FIELDS(V, _) \
+  V(KindField, CodeKind, 4, _) \
+  /* The other 12 bits are still free. */
+
+  DEFINE_BIT_FIELDS(FLAGS_BIT_FIELDS)
+#undef FLAGS_BIT_FIELDS
+  STATIC_ASSERT(FLAGS_BIT_FIELDS_Ranges::kBitsCount == 4);
+  STATIC_ASSERT(!V8_EXTERNAL_CODE_SPACE_BOOL ||
+                (FLAGS_BIT_FIELDS_Ranges::kBitsCount <=
+                 FIELD_SIZE(CodeDataContainer::kFlagsOffset) * kBitsPerByte));
+
  private:
   DECL_ACCESSORS(raw_code, Object)
-  DECL_RELAXED_ACCESSORS(raw_code, Object)
+  DECL_RELAXED_GETTER(raw_code, Object)
   inline void set_code_entry_point(Isolate* isolate, Address value);
 
+  // When V8_EXTERNAL_CODE_SPACE is enabled the flags field contains cached
+  // values of some flags of the from the respective Code object.
+  DECL_RELAXED_UINT16_ACCESSORS(flags)
+
   friend Factory;
+  friend FactoryBase<Factory>;
+  friend FactoryBase<LocalFactory>;
 
   OBJECT_CONSTRUCTORS(CodeDataContainer, HeapObject);
 };
@@ -273,8 +377,6 @@ class Code : public HeapObject {
 
   // [relocation_info]: Code relocation information
   DECL_ACCESSORS(relocation_info, ByteArray)
-  DECL_RELEASE_ACQUIRE_ACCESSORS(relocation_info, ByteArray)
-  DECL_ACCESSORS(relocation_info_or_undefined, HeapObject)
 
   // This function should be called only from GC.
   void ClearEmbeddedObjects(Heap* heap);
@@ -307,7 +409,6 @@ class Code : public HeapObject {
 
   // Unchecked accessors to be used during GC.
   inline ByteArray unchecked_relocation_info() const;
-  inline HeapObject synchronized_unchecked_relocation_info_or_undefined() const;
 
   inline int relocation_size() const;
 
@@ -335,6 +436,10 @@ class Code : public HeapObject {
   // TurboFan optimizing compiler.
   inline bool is_turbofanned() const;
 
+  // TODO(jgruber): Reconsider these predicates; we should probably merge them
+  // and rename to something appropriate.
+  inline bool is_maglevved() const;
+
   // [can_have_weak_objects]: If CodeKindIsOptimizedJSFunction(kind), tells
   // whether the embedded objects in code should be treated weakly.
   inline bool can_have_weak_objects() const;
@@ -350,10 +455,12 @@ class Code : public HeapObject {
   inline unsigned inlined_bytecode_size() const;
   inline void set_inlined_bytecode_size(unsigned size);
 
-  inline bool has_safepoint_info() const;
+  // [uses_safepoint_table]: Whether this Code object uses safepoint tables
+  // (note the table may still be empty, see has_safepoint_table).
+  inline bool uses_safepoint_table() const;
 
-  // [stack_slots]: If {has_safepoint_info()}, the number of stack slots
-  // reserved in the code prologue.
+  // [stack_slots]: If {uses_safepoint_table()}, the number of stack slots
+  // reserved in the code prologue; otherwise 0.
   inline int stack_slots() const;
 
   // [marked_for_deoptimization]: If CodeKindCanDeoptimize(kind), tells whether
@@ -361,13 +468,6 @@ class Code : public HeapObject {
   inline bool marked_for_deoptimization() const;
   inline void set_marked_for_deoptimization(bool flag);
 
-  // [deoptimization_count]: If CodeKindCanDeoptimize(kind). In turboprop we
-  // retain the deoptimized code on soft deopts for a certain number of soft
-  // deopts. This field keeps track of the number of deoptimizations we have
-  // seen so far.
-  inline int deoptimization_count() const;
-  inline void increment_deoptimization_count();
-
   // [embedded_objects_cleared]: If CodeKindIsOptimizedJSFunction(kind), tells
   // whether the embedded objects in the code marked for deoptimization were
   // cleared. Note that embedded_objects_cleared() implies
@@ -386,12 +486,6 @@ class Code : public HeapObject {
   // Use GetBuiltinCatchPrediction to access this.
   inline void set_is_promise_rejection(bool flag);
 
-  // [is_exception_caught]: For kind BUILTIN tells whether the
-  // exception thrown by the code will be caught internally or
-  // uncaught if both this and is_promise_rejection is set.
-  // Use GetBuiltinCatchPrediction to access this.
-  inline void set_is_exception_caught(bool flag);
-
   // [is_off_heap_trampoline]: For kind BUILTIN tells whether
   // this is a trampoline to an off-heap builtin.
   inline bool is_off_heap_trampoline() const;
@@ -408,6 +502,15 @@ class Code : public HeapObject {
   // out the to-be-overwritten header data for reproducible snapshots.
   inline void WipeOutHeader();
 
+  // When V8_EXTERNAL_CODE_SPACE is enabled, Code objects are allocated in
+  // a separate pointer compression cage instead of the cage where all the
+  // other objects are allocated.
+  // This field contains cage base value which is used for decompressing
+  // the references to non-Code objects (map, deoptimization_data, etc.).
+  inline PtrComprCageBase main_cage_base() const;
+  inline PtrComprCageBase main_cage_base(RelaxedLoadTag) const;
+  inline void set_main_cage_base(Address cage_base, RelaxedStoreTag);
+
   // Clear uninitialized padding space. This ensures that the snapshot content
   // is deterministic. Depending on the V8 build mode there could be no padding.
   inline void clear_padding();
@@ -446,10 +549,6 @@ class Code : public HeapObject {
   void CopyFromNoFlush(ByteArray reloc_info, Heap* heap, const CodeDesc& desc);
   void RelocateFromDesc(ByteArray reloc_info, Heap* heap, const CodeDesc& desc);
 
-#ifdef VERIFY_HEAP
-  void VerifyRelocInfo(Isolate* isolate, ByteArray reloc_info);
-#endif
-
   // Copy the RelocInfo portion of |desc| to |dest|. The ByteArray must be
   // exactly the same size as the RelocInfo in |desc|.
   static inline void CopyRelocInfoToByteArray(ByteArray dest,
@@ -481,11 +580,14 @@ class Code : public HeapObject {
     return RoundUp(kHeaderSize + body_size, kCodeAlignment);
   }
 
+  inline int CodeSize() const;
+
+  // Hides HeapObject::Size(...) and redirects queries to CodeSize().
+  DECL_GETTER(Size, int)
+
   DECL_CAST(Code)
 
   // Dispatched behavior.
-  inline int CodeSize() const;
-
   DECL_PRINTER(Code)
   DECL_VERIFIER(Code)
 
@@ -503,6 +605,8 @@ class Code : public HeapObject {
 
   static inline bool IsWeakObjectInOptimizedCode(HeapObject object);
 
+  static inline bool IsWeakObjectInDeoptimizationLiteralArray(Object object);
+
   // Returns false if this is an embedded builtin Code object that's in
   // read_only_space and hence doesn't have execute permissions.
   inline bool IsExecutable();
@@ -522,6 +626,8 @@ class Code : public HeapObject {
   /* The serializer needs to copy bytes starting from here verbatim. */       \
   /* Objects embedded into code is visited via reloc info. */                 \
   V(kDataStart, 0)                                                            \
+  V(kMainCageBaseUpper32BitsOffset,                                           \
+    V8_EXTERNAL_CODE_SPACE_BOOL ? kTaggedSize : 0)                            \
   V(kInstructionSizeOffset, kIntSize)                                         \
   V(kMetadataSizeOffset, kIntSize)                                            \
   V(kFlagsOffset, kInt32Size)                                                 \
@@ -545,13 +651,15 @@ class Code : public HeapObject {
   // This documents the amount of free space we have in each Code object header
   // due to padding for code alignment.
 #if V8_TARGET_ARCH_ARM64
-  static constexpr int kHeaderPaddingSize = COMPRESS_POINTERS_BOOL ? 12 : 24;
+  static constexpr int kHeaderPaddingSize =
+      V8_EXTERNAL_CODE_SPACE_BOOL ? 8 : (COMPRESS_POINTERS_BOOL ? 12 : 24);
 #elif V8_TARGET_ARCH_MIPS64
   static constexpr int kHeaderPaddingSize = 24;
 #elif V8_TARGET_ARCH_LOONG64
   static constexpr int kHeaderPaddingSize = 24;
 #elif V8_TARGET_ARCH_X64
-  static constexpr int kHeaderPaddingSize = COMPRESS_POINTERS_BOOL ? 12 : 56;
+  static constexpr int kHeaderPaddingSize =
+      V8_EXTERNAL_CODE_SPACE_BOOL ? 8 : (COMPRESS_POINTERS_BOOL ? 12 : 56);
 #elif V8_TARGET_ARCH_ARM
   static constexpr int kHeaderPaddingSize = 12;
 #elif V8_TARGET_ARCH_IA32
@@ -592,12 +700,10 @@ class Code : public HeapObject {
   V(EmbeddedObjectsClearedField, bool, 1, _)      \
   V(DeoptAlreadyCountedField, bool, 1, _)         \
   V(CanHaveWeakObjectsField, bool, 1, _)          \
-  V(IsPromiseRejectionField, bool, 1, _)          \
-  V(IsExceptionCaughtField, bool, 1, _)           \
-  V(DeoptCountField, int, 4, _)
+  V(IsPromiseRejectionField, bool, 1, _)
   DEFINE_BIT_FIELDS(CODE_KIND_SPECIFIC_FLAGS_BIT_FIELDS)
 #undef CODE_KIND_SPECIFIC_FLAGS_BIT_FIELDS
-  STATIC_ASSERT(CODE_KIND_SPECIFIC_FLAGS_BIT_FIELDS_Ranges::kBitsCount == 10);
+  STATIC_ASSERT(CODE_KIND_SPECIFIC_FLAGS_BIT_FIELDS_Ranges::kBitsCount == 5);
   STATIC_ASSERT(CODE_KIND_SPECIFIC_FLAGS_BIT_FIELDS_Ranges::kBitsCount <=
                 FIELD_SIZE(CodeDataContainer::kKindSpecificFlagsOffset) *
                     kBitsPerByte);
@@ -617,7 +723,6 @@ class Code : public HeapObject {
   inline CodeDataContainer GCSafeCodeDataContainer(AcquireLoadTag) const;
 
   bool is_promise_rejection() const;
-  bool is_exception_caught() const;
 
   enum BytecodeToPCPosition {
     kPcAtStartOfBytecode,
@@ -651,12 +756,14 @@ class Code::OptimizedCodeIterator {
 // Helper functions for converting Code objects to CodeDataContainer and back
 // when V8_EXTERNAL_CODE_SPACE is enabled.
 inline CodeT ToCodeT(Code code);
+inline Handle<CodeT> ToCodeT(Handle<Code> code, Isolate* isolate);
 inline Code FromCodeT(CodeT code);
 inline Code FromCodeT(CodeT code, RelaxedLoadTag);
 inline Code FromCodeT(CodeT code, AcquireLoadTag);
 inline Code FromCodeT(CodeT code, PtrComprCageBase);
 inline Code FromCodeT(CodeT code, PtrComprCageBase, RelaxedLoadTag);
 inline Code FromCodeT(CodeT code, PtrComprCageBase, AcquireLoadTag);
+inline Handle<CodeT> FromCodeT(Handle<Code> code, Isolate* isolate);
 inline CodeDataContainer CodeDataContainerFromCodeT(CodeT code);
 
 class AbstractCode : public HeapObject {
@@ -701,7 +808,7 @@ class AbstractCode : public HeapObject {
   inline int SizeIncludingMetadata();
 
   // Returns true if pc is inside this object's instructions.
-  inline bool contains(Address pc);
+  inline bool contains(Isolate* isolate, Address pc);
 
   // Returns the kind of the code.
   inline CodeKind kind();
@@ -720,34 +827,26 @@ class AbstractCode : public HeapObject {
   inline ByteArray SourcePositionTableInternal();
 };
 
-// Dependent code is a singly linked list of weak fixed arrays. Each array
-// contains weak pointers to code objects for one dependent group. The suffix of
-// the array can be filled with the undefined value if the number of codes is
-// less than the length of the array.
+// Dependent code is conceptually the list of {Code, DependencyGroup} tuples
+// associated with an object, where the dependency group is a reason that could
+// lead to a deopt of the corresponding code.
 //
-// +------+-----------------+--------+--------+-----+--------+-----------+-----+
-// | next | count & group 1 | code 1 | code 2 | ... | code n | undefined | ... |
-// +------+-----------------+--------+--------+-----+--------+-----------+-----+
-//    |
-//    V
-// +------+-----------------+--------+--------+-----+--------+-----------+-----+
-// | next | count & group 2 | code 1 | code 2 | ... | code m | undefined | ... |
-// +------+-----------------+--------+--------+-----+--------+-----------+-----+
-//    |
-//    V
-// empty_weak_fixed_array()
+// Implementation details: DependentCode is a weak array list containing
+// entries, where each entry consists of a (weak) Code object and the
+// DependencyGroups bitset as a Smi.
 //
-// The list of weak fixed arrays is ordered by dependency groups.
-
-class DependentCode : public WeakFixedArray {
+// Note the underlying weak array list currently never shrinks physically (the
+// contents may shrink).
+// TODO(jgruber): Consider adding physical shrinking.
+class DependentCode : public WeakArrayList {
  public:
   DECL_CAST(DependentCode)
 
   enum DependencyGroup {
-    // Group of code that embed a transition to this map, and depend on being
-    // deoptimized when the transition is replaced by a new version.
-    kTransitionGroup,
-    // Group of code that omit run-time prototype checks for prototypes
+    // Group of code objects that embed a transition to this map, and depend on
+    // being deoptimized when the transition is replaced by a new version.
+    kTransitionGroup = 1 << 0,
+    // Group of code objects that omit run-time prototype checks for prototypes
     // described by this map. The group is deoptimized whenever the following
     // conditions hold, possibly invalidating the assumptions embedded in the
     // code:
@@ -756,89 +855,82 @@ class DependentCode : public WeakFixedArray {
     // b) A dictionary-mode prototype described by this map changes shape, the
     // const-ness of one of its properties changes, or its [[Prototype]]
     // changes (only the latter causes a transition).
-    kPrototypeCheckGroup,
-    // Group of code that depends on global property values in property cells
-    // not being changed.
-    kPropertyCellChangedGroup,
-    // Group of code that omit run-time checks for field(s) introduced by
-    // this map, i.e. for the field type.
-    kFieldTypeGroup,
-    kFieldConstGroup,
-    kFieldRepresentationGroup,
-    // Group of code that omit run-time type checks for initial maps of
+    kPrototypeCheckGroup = 1 << 1,
+    // Group of code objects that depends on global property values in property
+    // cells not being changed.
+    kPropertyCellChangedGroup = 1 << 2,
+    // Group of code objects that omit run-time checks for field(s) introduced
+    // by this map, i.e. for the field type.
+    kFieldTypeGroup = 1 << 3,
+    kFieldConstGroup = 1 << 4,
+    kFieldRepresentationGroup = 1 << 5,
+    // Group of code objects that omit run-time type checks for initial maps of
     // constructors.
-    kInitialMapChangedGroup,
-    // Group of code that depends on tenuring information in AllocationSites
-    // not being changed.
-    kAllocationSiteTenuringChangedGroup,
-    // Group of code that depends on element transition information in
+    kInitialMapChangedGroup = 1 << 6,
+    // Group of code objects that depends on tenuring information in
     // AllocationSites not being changed.
-    kAllocationSiteTransitionChangedGroup
+    kAllocationSiteTenuringChangedGroup = 1 << 7,
+    // Group of code objects that depends on element transition information in
+    // AllocationSites not being changed.
+    kAllocationSiteTransitionChangedGroup = 1 << 8,
+    // IMPORTANT: The last bit must fit into a Smi, i.e. into 31 bits.
   };
+  using DependencyGroups = base::Flags<DependencyGroup, uint32_t>;
 
-  // Register a dependency of {code} on {object}, of the kind given by {group}.
+  static const char* DependencyGroupName(DependencyGroup group);
+
+  // Register a dependency of {code} on {object}, of the kinds given by
+  // {groups}.
   V8_EXPORT_PRIVATE static void InstallDependency(Isolate* isolate,
                                                   Handle<Code> code,
                                                   Handle<HeapObject> object,
-                                                  DependencyGroup group);
+                                                  DependencyGroups groups);
 
-  void DeoptimizeDependentCodeGroup(DependencyGroup group);
+  void DeoptimizeDependentCodeGroup(Isolate* isolate, DependencyGroups groups);
 
-  bool MarkCodeForDeoptimization(DependencyGroup group);
+  bool MarkCodeForDeoptimization(DependencyGroups deopt_groups);
 
-  // The following low-level accessors are exposed only for tests.
-  inline DependencyGroup group();
-  inline MaybeObject object_at(int i);
-  inline int count();
-  inline DependentCode next_link();
+  V8_EXPORT_PRIVATE static DependentCode empty_dependent_code(
+      const ReadOnlyRoots& roots);
+  static constexpr RootIndex kEmptyDependentCode =
+      RootIndex::kEmptyWeakArrayList;
+
+  // Constants exposed for tests.
+  static constexpr int kSlotsPerEntry = 2;  // {code: weak Code, groups: Smi}.
+  static constexpr int kCodeSlotOffset = 0;
+  static constexpr int kGroupsSlotOffset = 1;
 
  private:
-  static const char* DependencyGroupName(DependencyGroup group);
-
   // Get/Set {object}'s {DependentCode}.
   static DependentCode GetDependentCode(Handle<HeapObject> object);
   static void SetDependentCode(Handle<HeapObject> object,
                                Handle<DependentCode> dep);
 
-  static Handle<DependentCode> New(Isolate* isolate, DependencyGroup group,
-                                   Handle<Code> code,
-                                   Handle<DependentCode> next);
-  static Handle<DependentCode> EnsureSpace(Isolate* isolate,
-                                           Handle<DependentCode> entries);
+  static Handle<DependentCode> New(Isolate* isolate, DependencyGroups groups,
+                                   Handle<Code> code);
   static Handle<DependentCode> InsertWeakCode(Isolate* isolate,
                                               Handle<DependentCode> entries,
-                                              DependencyGroup group,
+                                              DependencyGroups groups,
                                               Handle<Code> code);
 
-  // Compact by removing cleared weak cells and return true if there was
-  // any cleared weak cell.
-  bool Compact();
+  // The callback is called for all non-cleared entries, and should return true
+  // iff the current entry should be cleared.
+  using IterateAndCompactFn = std::function<bool(CodeT, DependencyGroups)>;
+  void IterateAndCompact(const IterateAndCompactFn& fn);
 
-  static int Grow(int number_of_entries) {
-    if (number_of_entries < 5) return number_of_entries + 1;
-    return number_of_entries * 5 / 4;
+  // Fills the given entry with the last non-cleared entry in this list, and
+  // returns the new length after the last non-cleared entry has been moved.
+  int FillEntryFromBack(int index, int length);
+
+  static constexpr int LengthFor(int number_of_entries) {
+    return number_of_entries * kSlotsPerEntry;
   }
 
-  static const int kGroupCount = kAllocationSiteTransitionChangedGroup + 1;
-  static const int kNextLinkIndex = 0;
-  static const int kFlagsIndex = 1;
-  static const int kCodesStartIndex = 2;
-
-  inline void set_next_link(DependentCode next);
-  inline void set_count(int value);
-  inline void set_object_at(int i, MaybeObject object);
-  inline void clear_at(int i);
-  inline void copy(int from, int to);
-
-  inline int flags();
-  inline void set_flags(int flags);
-  using GroupField = base::BitField<int, 0, 5>;
-  using CountField = base::BitField<int, 5, 27>;
-  STATIC_ASSERT(kGroupCount <= GroupField::kMax + 1);
-
-  OBJECT_CONSTRUCTORS(DependentCode, WeakFixedArray);
+  OBJECT_CONSTRUCTORS(DependentCode, WeakArrayList);
 };
 
+DEFINE_OPERATORS_FOR_FLAGS(DependentCode::DependencyGroups)
+
 // BytecodeArray represents a sequence of interpreter bytecodes.
 class BytecodeArray
     : public TorqueGeneratedBytecodeArray<BytecodeArray, FixedArrayBase> {
@@ -950,6 +1042,24 @@ class BytecodeArray
   TQ_OBJECT_CONSTRUCTORS(BytecodeArray)
 };
 
+// This class holds data required during deoptimization. It does not have its
+// own instance type.
+class DeoptimizationLiteralArray : public WeakFixedArray {
+ public:
+  // Getters for literals. These include runtime checks that the pointer was not
+  // cleared, if the literal was held weakly.
+  inline Object get(int index) const;
+  inline Object get(PtrComprCageBase cage_base, int index) const;
+
+  // Setter for literals. This will set the object as strong or weak depending
+  // on Code::IsWeakObjectInOptimizedCode.
+  inline void set(int index, Object value);
+
+  DECL_CAST(DeoptimizationLiteralArray)
+
+  OBJECT_CONSTRUCTORS(DeoptimizationLiteralArray, WeakFixedArray);
+};
+
 // DeoptimizationData is a fixed array used to hold the deoptimization data for
 // optimized code.  It also contains information about functions that were
 // inlined.  If N different functions were inlined then the first N elements of
@@ -968,7 +1078,7 @@ class DeoptimizationData : public FixedArray {
   static const int kSharedFunctionInfoIndex = 6;
   static const int kInliningPositionsIndex = 7;
   static const int kDeoptExitStartIndex = 8;
-  static const int kEagerSoftAndBailoutDeoptCountIndex = 9;
+  static const int kNonLazyDeoptCountIndex = 9;
   static const int kLazyDeoptCountIndex = 10;
   static const int kFirstDeoptEntryIndex = 11;
 
@@ -990,14 +1100,14 @@ class DeoptimizationData : public FixedArray {
 
   DECL_ELEMENT_ACCESSORS(TranslationByteArray, TranslationArray)
   DECL_ELEMENT_ACCESSORS(InlinedFunctionCount, Smi)
-  DECL_ELEMENT_ACCESSORS(LiteralArray, FixedArray)
+  DECL_ELEMENT_ACCESSORS(LiteralArray, DeoptimizationLiteralArray)
   DECL_ELEMENT_ACCESSORS(OsrBytecodeOffset, Smi)
   DECL_ELEMENT_ACCESSORS(OsrPcOffset, Smi)
   DECL_ELEMENT_ACCESSORS(OptimizationId, Smi)
   DECL_ELEMENT_ACCESSORS(SharedFunctionInfo, Object)
   DECL_ELEMENT_ACCESSORS(InliningPositions, PodArray<InliningPosition>)
   DECL_ELEMENT_ACCESSORS(DeoptExitStart, Smi)
-  DECL_ELEMENT_ACCESSORS(EagerSoftAndBailoutDeoptCount, Smi)
+  DECL_ELEMENT_ACCESSORS(NonLazyDeoptCount, Smi)
   DECL_ELEMENT_ACCESSORS(LazyDeoptCount, Smi)
 
 #undef DECL_ELEMENT_ACCESSORS
