@@ -4,15 +4,24 @@
 
 #include "src/snapshot/code-serializer.h"
 
+#include <memory>
+
+#include "src/base/logging.h"
+#include "src/base/platform/elapsed-timer.h"
 #include "src/base/platform/platform.h"
 #include "src/codegen/macro-assembler.h"
 #include "src/common/globals.h"
 #include "src/debug/debug.h"
+#include "src/handles/maybe-handles.h"
+#include "src/handles/persistent-handles.h"
 #include "src/heap/heap-inl.h"
 #include "src/heap/local-factory-inl.h"
-#include "src/logging/counters.h"
+#include "src/heap/parked-scope.h"
+#include "src/logging/counters-scopes.h"
 #include "src/logging/log.h"
+#include "src/logging/runtime-call-stats-scope.h"
 #include "src/objects/objects-inl.h"
+#include "src/objects/shared-function-info.h"
 #include "src/objects/slots.h"
 #include "src/objects/visitors.h"
 #include "src/snapshot/object-deserializer.h"
@@ -23,7 +32,7 @@
 namespace v8 {
 namespace internal {
 
-ScriptData::ScriptData(const byte* data, int length)
+AlignedCachedData::AlignedCachedData(const byte* data, int length)
     : owns_data_(false), rejected_(false), data_(data), length_(length) {
   if (!IsAligned(reinterpret_cast<intptr_t>(data), kPointerAlignment)) {
     byte* copy = NewArray<byte>(length);
@@ -43,9 +52,9 @@ ScriptCompiler::CachedData* CodeSerializer::Serialize(
     Handle<SharedFunctionInfo> info) {
   Isolate* isolate = info->GetIsolate();
   TRACE_EVENT_CALL_STATS_SCOPED(isolate, "v8", "V8.Execute");
-  HistogramTimerScope histogram_timer(isolate->counters()->compile_serialize());
-  RuntimeCallTimerScope runtimeTimer(isolate,
-                                     RuntimeCallCounterId::kCompileSerialize);
+  NestedTimedHistogramScope histogram_timer(
+      isolate->counters()->compile_serialize());
+  RCS_SCOPE(isolate, RuntimeCallCounterId::kCompileSerialize);
   TRACE_EVENT0(TRACE_DISABLED_BY_DEFAULT("v8.compile"), "V8.CompileSerialize");
 
   base::ElapsedTimer timer;
@@ -56,9 +65,11 @@ ScriptCompiler::CachedData* CodeSerializer::Serialize(
     script->name().ShortPrint();
     PrintF("]\n");
   }
+#if V8_ENABLE_WEBASSEMBLY
   // TODO(7110): Enable serialization of Asm modules once the AsmWasmData is
   // context independent.
   if (script->ContainsAsmModule()) return nullptr;
+#endif  // V8_ENABLE_WEBASSEMBLY
 
   // Serialize code object.
   Handle<String> source(String::cast(script->source()), isolate);
@@ -67,24 +78,24 @@ ScriptCompiler::CachedData* CodeSerializer::Serialize(
                                  source, script->origin_options()));
   DisallowGarbageCollection no_gc;
   cs.reference_map()->AddAttachedReference(*source);
-  ScriptData* script_data = cs.SerializeSharedFunctionInfo(info);
+  AlignedCachedData* cached_data = cs.SerializeSharedFunctionInfo(info);
 
   if (FLAG_profile_deserialization) {
     double ms = timer.Elapsed().InMillisecondsF();
-    int length = script_data->length();
+    int length = cached_data->length();
     PrintF("[Serializing to %d bytes took %0.3f ms]\n", length, ms);
   }
 
   ScriptCompiler::CachedData* result =
-      new ScriptCompiler::CachedData(script_data->data(), script_data->length(),
+      new ScriptCompiler::CachedData(cached_data->data(), cached_data->length(),
                                      ScriptCompiler::CachedData::BufferOwned);
-  script_data->ReleaseDataOwnership();
-  delete script_data;
+  cached_data->ReleaseDataOwnership();
+  delete cached_data;
 
   return result;
 }
 
-ScriptData* CodeSerializer::SerializeSharedFunctionInfo(
+AlignedCachedData* CodeSerializer::SerializeSharedFunctionInfo(
     Handle<SharedFunctionInfo> info) {
   DisallowGarbageCollection no_gc;
 
@@ -128,7 +139,7 @@ void CodeSerializer::SerializeObjectImpl(Handle<HeapObject> obj) {
 
   if (SerializeReadOnlyObject(obj)) return;
 
-  CHECK(!obj->IsCode());
+  CHECK(!obj->IsCode(cage_base()));
 
   ReadOnlyRoots roots(isolate());
   if (ElideObject(*obj)) {
@@ -158,9 +169,12 @@ void CodeSerializer::SerializeObjectImpl(Handle<HeapObject> obj) {
 
   if (obj->IsSharedFunctionInfo()) {
     Handle<SharedFunctionInfo> sfi = Handle<SharedFunctionInfo>::cast(obj);
+    DCHECK(!sfi->IsApiFunction());
+#if V8_ENABLE_WEBASSEMBLY
     // TODO(7110): Enable serializing of Asm modules once the AsmWasmData
     // is context independent.
-    DCHECK(!sfi->IsApiFunction() && !sfi->HasAsmWasmData());
+    DCHECK(!sfi->HasAsmWasmData());
+#endif  // V8_ENABLE_WEBASSEMBLY
 
     DebugInfo debug_info;
     BytecodeArray debug_bytecode_array;
@@ -187,6 +201,25 @@ void CodeSerializer::SerializeObjectImpl(Handle<HeapObject> obj) {
     return;
   }
 
+  if (obj->IsUncompiledDataWithoutPreparseDataWithJob()) {
+    Handle<UncompiledDataWithoutPreparseDataWithJob> data =
+        Handle<UncompiledDataWithoutPreparseDataWithJob>::cast(obj);
+    Address job = data->job();
+    data->set_job(kNullAddress);
+    SerializeGeneric(data);
+    data->set_job(job);
+    return;
+  }
+  if (obj->IsUncompiledDataWithPreparseDataAndJob()) {
+    Handle<UncompiledDataWithPreparseDataAndJob> data =
+        Handle<UncompiledDataWithPreparseDataAndJob>::cast(obj);
+    Address job = data->job();
+    data->set_job(kNullAddress);
+    SerializeGeneric(data);
+    data->set_job(job);
+    return;
+  }
+
   // NOTE(mmarchini): If we try to serialize an InterpreterData our process
   // will crash since it stores a code object. Instead, we serialize the
   // bytecode array stored within the InterpreterData, which is the important
@@ -204,7 +237,8 @@ void CodeSerializer::SerializeObjectImpl(Handle<HeapObject> obj) {
   // There should be no references to the global object embedded.
   CHECK(!obj->IsJSGlobalProxy() && !obj->IsJSGlobalObject());
   // Embedded FixedArrays that need rehashing must support rehashing.
-  CHECK_IMPLIES(obj->NeedsRehashing(), obj->CanBeRehashed());
+  CHECK_IMPLIES(obj->NeedsRehashing(cage_base()),
+                obj->CanBeRehashed(cage_base()));
   // We expect no instantiated function objects or contexts.
   CHECK(!obj->IsJSFunction() && !obj->IsContext());
 
@@ -217,6 +251,8 @@ void CodeSerializer::SerializeGeneric(Handle<HeapObject> heap_object) {
   serializer.Serialize();
 }
 
+namespace {
+
 #ifndef V8_TARGET_ARCH_ARM
 // NOTE(mmarchini): when FLAG_interpreted_frames_native_stack is on, we want to
 // create duplicates of InterpreterEntryTrampoline for the deserialized
@@ -243,7 +279,7 @@ void CreateInterpreterDataForDeserializedCode(Isolate* isolate,
             INTERPRETER_DATA_TYPE, AllocationType::kOld));
 
     interpreter_data->set_bytecode_array(info->GetBytecodeArray(isolate));
-    interpreter_data->set_interpreter_trampoline(*code);
+    interpreter_data->set_interpreter_trampoline(ToCodeT(*code));
 
     info->set_interpreter_data(*interpreter_data);
 
@@ -252,96 +288,46 @@ void CreateInterpreterDataForDeserializedCode(Isolate* isolate,
     int line_num = script->GetLineNumber(info->StartPosition()) + 1;
     int column_num = script->GetColumnNumber(info->StartPosition()) + 1;
     PROFILE(isolate,
-            CodeCreateEvent(CodeEventListener::INTERPRETED_FUNCTION_TAG,
-                            abstract_code, info, name_handle, line_num,
-                            column_num));
+            CodeCreateEvent(CodeEventListener::FUNCTION_TAG, abstract_code,
+                            info, name_handle, line_num, column_num));
   }
 }
 #endif  // V8_TARGET_ARCH_ARM
 
-namespace {
 class StressOffThreadDeserializeThread final : public base::Thread {
  public:
   explicit StressOffThreadDeserializeThread(Isolate* isolate,
-                                            const SerializedCodeData* scd)
+                                            AlignedCachedData* cached_data)
       : Thread(
             base::Thread::Options("StressOffThreadDeserializeThread", 2 * MB)),
         isolate_(isolate),
-        scd_(scd) {}
-
-  MaybeHandle<SharedFunctionInfo> maybe_result() const { return maybe_result_; }
+        cached_data_(cached_data) {}
 
   void Run() final {
     LocalIsolate local_isolate(isolate_, ThreadKind::kBackground);
-    MaybeHandle<SharedFunctionInfo> local_maybe_result =
-        ObjectDeserializer::DeserializeSharedFunctionInfoOffThread(
-            &local_isolate, scd_, local_isolate.factory()->empty_string());
+    UnparkedScope unparked_scope(&local_isolate);
+    LocalHandleScope handle_scope(&local_isolate);
+    off_thread_data_ =
+        CodeSerializer::StartDeserializeOffThread(&local_isolate, cached_data_);
+  }
 
-    maybe_result_ =
-        local_isolate.heap()->NewPersistentMaybeHandle(local_maybe_result);
+  MaybeHandle<SharedFunctionInfo> Finalize(Isolate* isolate,
+                                           Handle<String> source,
+                                           ScriptOriginOptions origin_options) {
+    return CodeSerializer::FinishOffThreadDeserialize(
+        isolate, std::move(off_thread_data_), cached_data_, source,
+        origin_options);
   }
 
  private:
   Isolate* isolate_;
-  const SerializedCodeData* scd_;
-  MaybeHandle<SharedFunctionInfo> maybe_result_;
+  AlignedCachedData* cached_data_;
+  CodeSerializer::OffThreadDeserializeData off_thread_data_;
 };
-}  // namespace
-
-MaybeHandle<SharedFunctionInfo> CodeSerializer::Deserialize(
-    Isolate* isolate, ScriptData* cached_data, Handle<String> source,
-    ScriptOriginOptions origin_options) {
-  base::ElapsedTimer timer;
-  if (FLAG_profile_deserialization || FLAG_log_function_events) timer.Start();
-
-  HandleScope scope(isolate);
-
-  SerializedCodeData::SanityCheckResult sanity_check_result =
-      SerializedCodeData::CHECK_SUCCESS;
-  const SerializedCodeData scd = SerializedCodeData::FromCachedData(
-      cached_data, SerializedCodeData::SourceHash(source, origin_options),
-      &sanity_check_result);
-  if (sanity_check_result != SerializedCodeData::CHECK_SUCCESS) {
-    if (FLAG_profile_deserialization) PrintF("[Cached code failed check]\n");
-    DCHECK(cached_data->rejected());
-    isolate->counters()->code_cache_reject_reason()->AddSample(
-        sanity_check_result);
-    return MaybeHandle<SharedFunctionInfo>();
-  }
-
-  // Deserialize.
-  MaybeHandle<SharedFunctionInfo> maybe_result;
-  // TODO(leszeks): Add LocalHeap support to deserializer
-  if (false && FLAG_stress_background_compile) {
-    StressOffThreadDeserializeThread thread(isolate, &scd);
-    CHECK(thread.Start());
-    thread.Join();
-
-    maybe_result = thread.maybe_result();
-
-    // Fix-up result script source.
-    Handle<SharedFunctionInfo> result;
-    if (maybe_result.ToHandle(&result)) {
-      Script::cast(result->script()).set_source(*source);
-    }
-  } else {
-    maybe_result = ObjectDeserializer::DeserializeSharedFunctionInfo(
-        isolate, &scd, source);
-  }
-
-  Handle<SharedFunctionInfo> result;
-  if (!maybe_result.ToHandle(&result)) {
-    // Deserializing may fail if the reservations cannot be fulfilled.
-    if (FLAG_profile_deserialization) PrintF("[Deserializing failed]\n");
-    return MaybeHandle<SharedFunctionInfo>();
-  }
-
-  if (FLAG_profile_deserialization) {
-    double ms = timer.Elapsed().InMillisecondsF();
-    int length = cached_data->length();
-    PrintF("[Deserializing from %d bytes took %0.3f ms]\n", length, ms);
-  }
 
+void FinalizeDeserialization(Isolate* isolate,
+                             Handle<SharedFunctionInfo> result,
+                             const base::ElapsedTimer& timer) {
   const bool log_code_creation =
       isolate->logger()->is_listening_to_code_events() ||
       isolate->is_profiling() ||
@@ -385,11 +371,13 @@ MaybeHandle<SharedFunctionInfo> CodeSerializer::Deserialize(
               script->GetLineNumber(shared_info->StartPosition()) + 1;
           int column_num =
               script->GetColumnNumber(shared_info->StartPosition()) + 1;
-          PROFILE(isolate,
-                  CodeCreateEvent(
-                      CodeEventListener::SCRIPT_TAG,
-                      handle(shared_info->abstract_code(isolate), isolate),
-                      shared_info, name, line_num, column_num));
+          PROFILE(
+              isolate,
+              CodeCreateEvent(
+                  shared_info->is_toplevel() ? CodeEventListener::SCRIPT_TAG
+                                             : CodeEventListener::FUNCTION_TAG,
+                  handle(shared_info->abstract_code(isolate), isolate),
+                  shared_info, name, line_num, column_num));
         }
       }
     }
@@ -399,6 +387,166 @@ MaybeHandle<SharedFunctionInfo> CodeSerializer::Deserialize(
     Handle<Script> script(Script::cast(result->script()), isolate);
     Script::InitLineEnds(isolate, script);
   }
+}
+
+}  // namespace
+
+MaybeHandle<SharedFunctionInfo> CodeSerializer::Deserialize(
+    Isolate* isolate, AlignedCachedData* cached_data, Handle<String> source,
+    ScriptOriginOptions origin_options) {
+  if (FLAG_stress_background_compile) {
+    StressOffThreadDeserializeThread thread(isolate, cached_data);
+    CHECK(thread.Start());
+    thread.Join();
+    return thread.Finalize(isolate, source, origin_options);
+    // TODO(leszeks): Compare off-thread deserialized data to on-thread.
+  }
+
+  base::ElapsedTimer timer;
+  if (FLAG_profile_deserialization || FLAG_log_function_events) timer.Start();
+
+  HandleScope scope(isolate);
+
+  SerializedCodeSanityCheckResult sanity_check_result =
+      SerializedCodeSanityCheckResult::kSuccess;
+  const SerializedCodeData scd = SerializedCodeData::FromCachedData(
+      cached_data, SerializedCodeData::SourceHash(source, origin_options),
+      &sanity_check_result);
+  if (sanity_check_result != SerializedCodeSanityCheckResult::kSuccess) {
+    if (FLAG_profile_deserialization) PrintF("[Cached code failed check]\n");
+    DCHECK(cached_data->rejected());
+    isolate->counters()->code_cache_reject_reason()->AddSample(
+        static_cast<int>(sanity_check_result));
+    return MaybeHandle<SharedFunctionInfo>();
+  }
+
+  // Deserialize.
+  MaybeHandle<SharedFunctionInfo> maybe_result =
+      ObjectDeserializer::DeserializeSharedFunctionInfo(isolate, &scd, source);
+
+  Handle<SharedFunctionInfo> result;
+  if (!maybe_result.ToHandle(&result)) {
+    // Deserializing may fail if the reservations cannot be fulfilled.
+    if (FLAG_profile_deserialization) PrintF("[Deserializing failed]\n");
+    return MaybeHandle<SharedFunctionInfo>();
+  }
+
+  if (FLAG_profile_deserialization) {
+    double ms = timer.Elapsed().InMillisecondsF();
+    int length = cached_data->length();
+    PrintF("[Deserializing from %d bytes took %0.3f ms]\n", length, ms);
+  }
+
+  FinalizeDeserialization(isolate, result, timer);
+
+  return scope.CloseAndEscape(result);
+}
+
+CodeSerializer::OffThreadDeserializeData
+CodeSerializer::StartDeserializeOffThread(LocalIsolate* local_isolate,
+                                          AlignedCachedData* cached_data) {
+  OffThreadDeserializeData result;
+
+  DCHECK(!local_isolate->heap()->HasPersistentHandles());
+
+  const SerializedCodeData scd =
+      SerializedCodeData::FromCachedDataWithoutSource(
+          cached_data, &result.sanity_check_result);
+  if (result.sanity_check_result != SerializedCodeSanityCheckResult::kSuccess) {
+    // Exit early but don't report yet, we'll re-check this when finishing on
+    // the main thread
+    DCHECK(cached_data->rejected());
+    return result;
+  }
+
+  MaybeHandle<SharedFunctionInfo> local_maybe_result =
+      OffThreadObjectDeserializer::DeserializeSharedFunctionInfo(
+          local_isolate, &scd, &result.scripts);
+
+  result.maybe_result =
+      local_isolate->heap()->NewPersistentMaybeHandle(local_maybe_result);
+  result.persistent_handles = local_isolate->heap()->DetachPersistentHandles();
+
+  return result;
+}
+
+MaybeHandle<SharedFunctionInfo> CodeSerializer::FinishOffThreadDeserialize(
+    Isolate* isolate, OffThreadDeserializeData&& data,
+    AlignedCachedData* cached_data, Handle<String> source,
+    ScriptOriginOptions origin_options) {
+  base::ElapsedTimer timer;
+  if (FLAG_profile_deserialization || FLAG_log_function_events) timer.Start();
+
+  HandleScope scope(isolate);
+
+  // Do a source sanity check now that we have the source. It's important for
+  // FromPartiallySanityCheckedCachedData call that the sanity_check_result
+  // holds the result of the off-thread sanity check.
+  SerializedCodeSanityCheckResult sanity_check_result =
+      data.sanity_check_result;
+  const SerializedCodeData scd =
+      SerializedCodeData::FromPartiallySanityCheckedCachedData(
+          cached_data, SerializedCodeData::SourceHash(source, origin_options),
+          &sanity_check_result);
+  if (sanity_check_result != SerializedCodeSanityCheckResult::kSuccess) {
+    // The only case where the deserialization result could exist despite a
+    // check failure is on a source mismatch, since we can't test for this
+    // off-thread.
+    DCHECK_IMPLIES(!data.maybe_result.is_null(),
+                   sanity_check_result ==
+                       SerializedCodeSanityCheckResult::kSourceMismatch);
+    // The only kind of sanity check we can't test for off-thread is a source
+    // mismatch.
+    DCHECK_IMPLIES(sanity_check_result != data.sanity_check_result,
+                   sanity_check_result ==
+                       SerializedCodeSanityCheckResult::kSourceMismatch);
+    if (FLAG_profile_deserialization) PrintF("[Cached code failed check]\n");
+    DCHECK(cached_data->rejected());
+    isolate->counters()->code_cache_reject_reason()->AddSample(
+        static_cast<int>(sanity_check_result));
+    return MaybeHandle<SharedFunctionInfo>();
+  }
+
+  Handle<SharedFunctionInfo> result;
+  if (!data.maybe_result.ToHandle(&result)) {
+    // Deserializing may fail if the reservations cannot be fulfilled.
+    if (FLAG_profile_deserialization) {
+      PrintF("[Off-thread deserializing failed]\n");
+    }
+    return MaybeHandle<SharedFunctionInfo>();
+  }
+
+  // Change the result persistent handle into a regular handle.
+  DCHECK(data.persistent_handles->Contains(result.location()));
+  result = handle(*result, isolate);
+
+  // Fix up the source on the script. This should be the only deserialized
+  // script, and the off-thread deserializer should have set its source to
+  // the empty string.
+  DCHECK_EQ(data.scripts.size(), 1);
+  DCHECK_EQ(result->script(), *data.scripts[0]);
+  DCHECK_EQ(Script::cast(result->script()).source(),
+            ReadOnlyRoots(isolate).empty_string());
+  Script::cast(result->script()).set_source(*source);
+
+  // Fix up the script list to include the newly deserialized script.
+  Handle<WeakArrayList> list = isolate->factory()->script_list();
+  for (Handle<Script> script : data.scripts) {
+    DCHECK(data.persistent_handles->Contains(script.location()));
+    list =
+        WeakArrayList::AddToEnd(isolate, list, MaybeObjectHandle::Weak(script));
+  }
+  isolate->heap()->SetRootScriptList(*list);
+
+  if (FLAG_profile_deserialization) {
+    double ms = timer.Elapsed().InMillisecondsF();
+    int length = cached_data->length();
+    PrintF("[Finishing off-thread deserialize from %d bytes took %0.3f ms]\n",
+           length, ms);
+  }
+
+  FinalizeDeserialization(isolate, result, timer);
+
   return scope.CloseAndEscape(result);
 }
 
@@ -429,27 +577,56 @@ SerializedCodeData::SerializedCodeData(const std::vector<byte>* payload,
   // Copy serialized data.
   CopyBytes(data_ + kHeaderSize, payload->data(),
             static_cast<size_t>(payload->size()));
-
-  SetHeaderValue(kChecksumOffset, Checksum(ChecksummedContent()));
+  uint32_t checksum =
+      FLAG_verify_snapshot_checksum ? Checksum(ChecksummedContent()) : 0;
+  SetHeaderValue(kChecksumOffset, checksum);
 }
 
-SerializedCodeData::SanityCheckResult SerializedCodeData::SanityCheck(
+SerializedCodeSanityCheckResult SerializedCodeData::SanityCheck(
+    uint32_t expected_source_hash) const {
+  SerializedCodeSanityCheckResult result = SanityCheckWithoutSource();
+  if (result != SerializedCodeSanityCheckResult::kSuccess) return result;
+  return SanityCheckJustSource(expected_source_hash);
+}
+
+SerializedCodeSanityCheckResult SerializedCodeData::SanityCheckJustSource(
     uint32_t expected_source_hash) const {
-  if (this->size_ < kHeaderSize) return INVALID_HEADER;
-  uint32_t magic_number = GetMagicNumber();
-  if (magic_number != kMagicNumber) return MAGIC_NUMBER_MISMATCH;
-  uint32_t version_hash = GetHeaderValue(kVersionHashOffset);
   uint32_t source_hash = GetHeaderValue(kSourceHashOffset);
+  if (source_hash != expected_source_hash) {
+    return SerializedCodeSanityCheckResult::kSourceMismatch;
+  }
+  return SerializedCodeSanityCheckResult::kSuccess;
+}
+
+SerializedCodeSanityCheckResult SerializedCodeData::SanityCheckWithoutSource()
+    const {
+  if (this->size_ < kHeaderSize) {
+    return SerializedCodeSanityCheckResult::kInvalidHeader;
+  }
+  uint32_t magic_number = GetMagicNumber();
+  if (magic_number != kMagicNumber) {
+    return SerializedCodeSanityCheckResult::kMagicNumberMismatch;
+  }
+  uint32_t version_hash = GetHeaderValue(kVersionHashOffset);
+  if (version_hash != Version::Hash()) {
+    return SerializedCodeSanityCheckResult::kVersionMismatch;
+  }
   uint32_t flags_hash = GetHeaderValue(kFlagHashOffset);
+  if (flags_hash != FlagList::Hash()) {
+    return SerializedCodeSanityCheckResult::kFlagsMismatch;
+  }
   uint32_t payload_length = GetHeaderValue(kPayloadLengthOffset);
-  uint32_t c = GetHeaderValue(kChecksumOffset);
-  if (version_hash != Version::Hash()) return VERSION_MISMATCH;
-  if (source_hash != expected_source_hash) return SOURCE_MISMATCH;
-  if (flags_hash != FlagList::Hash()) return FLAGS_MISMATCH;
   uint32_t max_payload_length = this->size_ - kHeaderSize;
-  if (payload_length > max_payload_length) return LENGTH_MISMATCH;
-  if (Checksum(ChecksummedContent()) != c) return CHECKSUM_MISMATCH;
-  return CHECK_SUCCESS;
+  if (payload_length > max_payload_length) {
+    return SerializedCodeSanityCheckResult::kLengthMismatch;
+  }
+  if (FLAG_verify_snapshot_checksum) {
+    uint32_t checksum = GetHeaderValue(kChecksumOffset);
+    if (Checksum(ChecksummedContent()) != checksum) {
+      return SerializedCodeSanityCheckResult::kChecksumMismatch;
+    }
+  }
+  return SerializedCodeSanityCheckResult::kSuccess;
 }
 
 uint32_t SerializedCodeData::SourceHash(Handle<String> source,
@@ -464,33 +641,74 @@ uint32_t SerializedCodeData::SourceHash(Handle<String> source,
 }
 
 // Return ScriptData object and relinquish ownership over it to the caller.
-ScriptData* SerializedCodeData::GetScriptData() {
+AlignedCachedData* SerializedCodeData::GetScriptData() {
   DCHECK(owns_data_);
-  ScriptData* result = new ScriptData(data_, size_);
+  AlignedCachedData* result = new AlignedCachedData(data_, size_);
   result->AcquireDataOwnership();
   owns_data_ = false;
   data_ = nullptr;
   return result;
 }
 
-Vector<const byte> SerializedCodeData::Payload() const {
+base::Vector<const byte> SerializedCodeData::Payload() const {
   const byte* payload = data_ + kHeaderSize;
   DCHECK(IsAligned(reinterpret_cast<intptr_t>(payload), kPointerAlignment));
   int length = GetHeaderValue(kPayloadLengthOffset);
   DCHECK_EQ(data_ + size_, payload + length);
-  return Vector<const byte>(payload, length);
+  return base::Vector<const byte>(payload, length);
 }
 
-SerializedCodeData::SerializedCodeData(ScriptData* data)
+SerializedCodeData::SerializedCodeData(AlignedCachedData* data)
     : SerializedData(const_cast<byte*>(data->data()), data->length()) {}
 
 SerializedCodeData SerializedCodeData::FromCachedData(
-    ScriptData* cached_data, uint32_t expected_source_hash,
-    SanityCheckResult* rejection_result) {
+    AlignedCachedData* cached_data, uint32_t expected_source_hash,
+    SerializedCodeSanityCheckResult* rejection_result) {
   DisallowGarbageCollection no_gc;
   SerializedCodeData scd(cached_data);
   *rejection_result = scd.SanityCheck(expected_source_hash);
-  if (*rejection_result != CHECK_SUCCESS) {
+  if (*rejection_result != SerializedCodeSanityCheckResult::kSuccess) {
+    cached_data->Reject();
+    return SerializedCodeData(nullptr, 0);
+  }
+  return scd;
+}
+
+SerializedCodeData SerializedCodeData::FromCachedDataWithoutSource(
+    AlignedCachedData* cached_data,
+    SerializedCodeSanityCheckResult* rejection_result) {
+  DisallowGarbageCollection no_gc;
+  SerializedCodeData scd(cached_data);
+  *rejection_result = scd.SanityCheckWithoutSource();
+  if (*rejection_result != SerializedCodeSanityCheckResult::kSuccess) {
+    cached_data->Reject();
+    return SerializedCodeData(nullptr, 0);
+  }
+  return scd;
+}
+
+SerializedCodeData SerializedCodeData::FromPartiallySanityCheckedCachedData(
+    AlignedCachedData* cached_data, uint32_t expected_source_hash,
+    SerializedCodeSanityCheckResult* rejection_result) {
+  DisallowGarbageCollection no_gc;
+  // The previous call to FromCachedDataWithoutSource may have already rejected
+  // the cached data, so re-use the previous rejection result if it's not a
+  // success.
+  if (*rejection_result != SerializedCodeSanityCheckResult::kSuccess) {
+    // FromCachedDataWithoutSource doesn't check the source, so there can't be
+    // a source mismatch.
+    DCHECK_NE(*rejection_result,
+              SerializedCodeSanityCheckResult::kSourceMismatch);
+    cached_data->Reject();
+    return SerializedCodeData(nullptr, 0);
+  }
+  SerializedCodeData scd(cached_data);
+  *rejection_result = scd.SanityCheckJustSource(expected_source_hash);
+  if (*rejection_result != SerializedCodeSanityCheckResult::kSuccess) {
+    // This check only checks the source, so the only possible failure is a
+    // source mismatch.
+    DCHECK_EQ(*rejection_result,
+              SerializedCodeSanityCheckResult::kSourceMismatch);
     cached_data->Reject();
     return SerializedCodeData(nullptr, 0);
   }
