@@ -148,9 +148,11 @@ class V8_EXPORT_PRIVATE WasmEngine {
   ~WasmEngine();
 
   // Synchronously validates the given bytes that represent an encoded Wasm
-  // module.
+  // module. If validation fails and {error_msg} is present, it is set to the
+  // validation error.
   bool SyncValidate(Isolate* isolate, const WasmFeatures& enabled,
-                    const ModuleWireBytes& bytes);
+                    const ModuleWireBytes& bytes,
+                    std::string* error_message = nullptr);
 
   // Synchronously compiles the given bytes that represent a translated
   // asm.js module.
@@ -224,6 +226,8 @@ class V8_EXPORT_PRIVATE WasmEngine {
 
   // Prints the gathered compilation statistics, then resets them.
   void DumpAndResetTurboStatistics();
+  // Same, but no reset.
+  void DumpTurboStatistics();
 
   // Used to redirect tracing output from {stdout} to a file.
   CodeTracer* GetCodeTracer();
