@@ -12,7 +12,7 @@
 #include "src/base/memory.h"
 #include "src/handles/handles.h"
 #include "src/utils/boxed-float.h"
-#include "src/wasm/wasm-opcodes.h"
+#include "src/wasm/value-type.h"
 #include "src/zone/zone-containers.h"
 
 namespace v8 {
@@ -42,16 +42,21 @@ class Simd128 {
   explicit Simd128(sType val) {                                              \
     base::WriteUnalignedValue<sType>(reinterpret_cast<Address>(val_), val);  \
   }                                                                          \
-  sType to_##name() {                                                        \
+  sType to_##name() const {                                                  \
     return base::ReadUnalignedValue<sType>(reinterpret_cast<Address>(val_)); \
   }
   FOREACH_SIMD_TYPE(DEFINE_SIMD_TYPE_SPECIFIC_METHODS)
 #undef DEFINE_SIMD_TYPE_SPECIFIC_METHODS
 
+  explicit Simd128(byte* bytes) {
+    memcpy(static_cast<void*>(val_), reinterpret_cast<void*>(bytes),
+           kSimd128Size);
+  }
+
   const uint8_t* bytes() { return val_; }
 
   template <typename T>
-  inline T to();
+  inline T to() const;
 
  private:
   uint8_t val_[16] = {0};
@@ -59,7 +64,7 @@ class Simd128 {
 
 #define DECLARE_CAST(cType, sType, name, size) \
   template <>                                  \
-  inline sType Simd128::to() {                 \
+  inline sType Simd128::to() const {           \
     return to_##name();                        \
   }
 FOREACH_SIMD_TYPE(DECLARE_CAST)
@@ -90,30 +95,38 @@ class WasmValue {
  public:
   WasmValue() : type_(kWasmVoid), bit_pattern_{} {}
 
-#define DEFINE_TYPE_SPECIFIC_METHODS(name, localtype, ctype)       \
-  explicit WasmValue(ctype v) : type_(localtype), bit_pattern_{} { \
-    static_assert(sizeof(ctype) <= sizeof(bit_pattern_),           \
-                  "size too big for WasmValue");                   \
-    base::WriteLittleEndianValue<ctype>(                           \
-        reinterpret_cast<Address>(bit_pattern_), v);               \
-  }                                                                \
-  ctype to_##name() const {                                        \
-    DCHECK_EQ(localtype, type_);                                   \
-    return to_##name##_unchecked();                                \
-  }                                                                \
-  ctype to_##name##_unchecked() const {                            \
-    return base::ReadLittleEndianValue<ctype>(                     \
-        reinterpret_cast<Address>(bit_pattern_));                  \
+#define DEFINE_TYPE_SPECIFIC_METHODS(name, localtype, ctype)                  \
+  explicit WasmValue(ctype v) : type_(localtype), bit_pattern_{} {            \
+    static_assert(sizeof(ctype) <= sizeof(bit_pattern_),                      \
+                  "size too big for WasmValue");                              \
+    base::WriteUnalignedValue<ctype>(reinterpret_cast<Address>(bit_pattern_), \
+                                     v);                                      \
+  }                                                                           \
+  ctype to_##name() const {                                                   \
+    DCHECK_EQ(localtype, type_);                                              \
+    return to_##name##_unchecked();                                           \
+  }                                                                           \
+  ctype to_##name##_unchecked() const {                                       \
+    return base::ReadUnalignedValue<ctype>(                                   \
+        reinterpret_cast<Address>(bit_pattern_));                             \
   }
+
   FOREACH_PRIMITIVE_WASMVAL_TYPE(DEFINE_TYPE_SPECIFIC_METHODS)
 #undef DEFINE_TYPE_SPECIFIC_METHODS
 
+  WasmValue(byte* raw_bytes, ValueType type) : type_(type), bit_pattern_{} {
+    DCHECK(type_.is_numeric());
+    memcpy(bit_pattern_, raw_bytes, type.element_size_bytes());
+  }
+
   WasmValue(Handle<Object> ref, ValueType type) : type_(type), bit_pattern_{} {
     static_assert(sizeof(Handle<Object>) <= sizeof(bit_pattern_),
                   "bit_pattern_ must be large enough to fit a Handle");
+    DCHECK(type.is_reference());
     base::WriteUnalignedValue<Handle<Object>>(
         reinterpret_cast<Address>(bit_pattern_), ref);
   }
+
   Handle<Object> to_ref() const {
     DCHECK(type_.is_reference());
     return base::ReadUnalignedValue<Handle<Object>>(
@@ -125,7 +138,30 @@ class WasmValue {
   // Checks equality of type and bit pattern (also for float and double values).
   bool operator==(const WasmValue& other) const {
     return type_ == other.type_ &&
-           !memcmp(bit_pattern_, other.bit_pattern_, 16);
+           !memcmp(bit_pattern_, other.bit_pattern_,
+                   type_.is_reference() ? sizeof(Handle<Object>)
+                                        : type_.element_size_bytes());
+  }
+
+  void CopyTo(byte* to) const {
+    STATIC_ASSERT(sizeof(float) == sizeof(Float32));
+    STATIC_ASSERT(sizeof(double) == sizeof(Float64));
+    DCHECK(type_.is_numeric());
+    memcpy(to, bit_pattern_, type_.element_size_bytes());
+  }
+
+  // If {packed_type.is_packed()}, create a new value of {packed_type()}.
+  // Otherwise, return this object.
+  WasmValue Packed(ValueType packed_type) const {
+    if (packed_type == kWasmI8) {
+      DCHECK_EQ(type_, kWasmI32);
+      return WasmValue(static_cast<int8_t>(to_i32()));
+    }
+    if (packed_type == kWasmI16) {
+      DCHECK_EQ(type_, kWasmI32);
+      return WasmValue(static_cast<int16_t>(to_i32()));
+    }
+    return *this;
   }
 
   template <typename T>
@@ -140,6 +176,39 @@ class WasmValue {
     return WasmValue{type{value}};
   }
 
+  inline std::string to_string() const {
+    switch (type_.kind()) {
+      case kI8:
+        return std::to_string(to_i8());
+      case kI16:
+        return std::to_string(to_i16());
+      case kI32:
+        return std::to_string(to_i32());
+      case kI64:
+        return std::to_string(to_i64());
+      case kF32:
+        return std::to_string(to_f32());
+      case kF64:
+        return std::to_string(to_f64());
+      case kS128: {
+        std::stringstream stream;
+        stream << "0x" << std::hex;
+        for (int8_t byte : bit_pattern_) {
+          if (!(byte & 0xf0)) stream << '0';
+          stream << byte;
+        }
+        return stream.str();
+      }
+      case kOptRef:
+      case kRef:
+      case kRtt:
+        return "Handle [" + std::to_string(to_ref().address()) + "]";
+      case kVoid:
+      case kBottom:
+        UNREACHABLE();
+    }
+  }
+
  private:
   ValueType type_;
   uint8_t bit_pattern_[16];
