@@ -30,7 +30,11 @@ import sys
 import tempfile
 import unittest
 
-from cStringIO import StringIO
+# TODO(https://crbug.com/1292016): Remove after Python3 migration.
+try:
+  from cStringIO import StringIO
+except ImportError:
+  from io import StringIO
 
 TOOLS_ROOT = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
 TEST_DATA_ROOT = os.path.join(TOOLS_ROOT, 'unittests', 'testdata')
@@ -277,7 +281,9 @@ class SystemTest(unittest.TestCase):
     # We need lexicographic sorting here to avoid non-deterministic behaviour
     # The original sorting key is duration, but in our fake test we have
     # non-deterministic durations before we reset them to 1
-    json_output['slowest_tests'].sort(key= lambda x: str(x))
+    def sort_key(x):
+      return str(sorted(x.items()))
+    json_output['slowest_tests'].sort(key=sort_key)
 
     with open(os.path.join(TEST_DATA_ROOT, expected_results_name)) as f:
       expected_test_results = json.load(f)
@@ -351,7 +357,8 @@ class SystemTest(unittest.TestCase):
           v8_enable_verify_csa=False, v8_enable_lite_mode=False,
           v8_enable_pointer_compression=False,
           v8_enable_pointer_compression_shared_cage=False,
-          v8_enable_virtual_memory_cage=False)
+          v8_enable_shared_ro_heap=False,
+          v8_enable_sandbox=False)
       result = run_tests(
           basedir,
           '--progress=verbose',
