@@ -409,10 +409,11 @@ function storeHTTPOptions(options) {
 function setupConnectionsTracking(server) {
   // Start connection handling
   server[kConnections] = new ConnectionsList();
-  if (server.headersTimeout > 0 || server.requestTimeout > 0) {
-    server[kConnectionsCheckingInterval] =
-      setInterval(checkConnections.bind(server), server.connectionsCheckingInterval).unref();
-  }
+
+  // This checker is started without checking whether any headersTimeout or requestTimeout is non zero
+  // otherwise it would not be started if such timeouts are modified after createServer.
+  server[kConnectionsCheckingInterval] =
+    setInterval(checkConnections.bind(server), server.connectionsCheckingInterval).unref();
 }
 
 function Server(options, requestListener) {
@@ -458,6 +459,22 @@ Server.prototype.close = function() {
   ReflectApply(net.Server.prototype.close, this, arguments);
 };
 
+Server.prototype.closeAllConnections = function() {
+  const connections = this[kConnections].all();
+
+  for (let i = 0, l = connections.length; i < l; i++) {
+    connections[i].socket.destroy();
+  }
+};
+
+Server.prototype.closeIdleConnections = function() {
+  const connections = this[kConnections].idle();
+
+  for (let i = 0, l = connections.length; i < l; i++) {
+    connections[i].socket.destroy();
+  }
+};
+
 Server.prototype.setTimeout = function setTimeout(msecs, callback) {
   this.timeout = msecs;
   if (callback)
@@ -489,6 +506,10 @@ Server.prototype[EE.captureRejectionSymbol] = function(err, event, ...args) {
 };
 
 function checkConnections() {
+  if (this.headersTimeout === 0 && this.requestTimeout === 0) {
+    return;
+  }
+
   const expired = this[kConnections].expired(this.headersTimeout, this.requestTimeout);
 
   for (let i = 0; i < expired.length; i++) {
