@@ -86,7 +86,6 @@ const {
 const {
   validateBoolean,
   validateBuffer,
-  validateCallback,
   validateFunction,
   validateInt32,
   validateNumber,
@@ -408,8 +407,10 @@ function onocspresponse(resp) {
 function onerror(err) {
   const owner = this[owner_symbol];
   debug('%s onerror %s had? %j',
-        owner._tlsOptions.isServer ? 'server' : 'client', err,
-        owner._hadError);
+        (typeof owner._tlsOptions === 'object' && owner._tlsOptions !== null) ?
+          owner._tlsOptions.isServer ? 'server' : 'client' :
+          'unknown',
+        err, owner._hadError);
 
   if (owner._hadError)
     return;
@@ -421,7 +422,7 @@ function onerror(err) {
     // When handshake fails control is not yet released,
     // so self._tlsError will return null instead of actual error
     owner.destroy(err);
-  } else if (owner._tlsOptions.isServer &&
+  } else if (owner._tlsOptions?.isServer &&
              owner._rejectUnauthorized &&
              RegExpPrototypeTest(/peer did not return a certificate/,
                                  err.message)) {
@@ -826,7 +827,7 @@ TLSSocket.prototype._init = function(socket, wrap) {
 TLSSocket.prototype.renegotiate = function(options, callback) {
   validateObject(options, 'options');
   if (callback !== undefined) {
-    validateCallback(callback);
+    validateFunction(callback, 'callback');
   }
 
   debug('%s renegotiate()',
