@@ -42,7 +42,6 @@ const {
   Resolver,
   validateHints,
   emitInvalidHostnameWarning,
-  emitTypeCoercionDeprecationWarning,
   getDefaultVerbatim,
   setDefaultResultOrder,
 } = require('internal/dns/utils');
@@ -52,10 +51,12 @@ const {
   ERR_MISSING_ARGS,
 } = errors.codes;
 const {
-  validateCallback,
+  validateBoolean,
+  validateFunction,
+  validateNumber,
+  validateOneOf,
   validatePort,
   validateString,
-  validateOneOf,
 } = require('internal/validators');
 
 const {
@@ -107,9 +108,10 @@ function onlookupall(err, addresses) {
 
 // Easy DNS A/AAAA look up
 // lookup(hostname, [options,] callback)
+const validFamilies = [0, 4, 6];
 function lookup(hostname, options, callback) {
   let hints = 0;
-  let family = -1;
+  let family = 0;
   let all = false;
   let verbatim = getDefaultVerbatim();
 
@@ -121,39 +123,36 @@ function lookup(hostname, options, callback) {
   if (typeof options === 'function') {
     callback = options;
     family = 0;
+  } else if (typeof options === 'number') {
+    validateFunction(callback, 'callback');
+
+    validateOneOf(options, 'family', validFamilies, true);
+    family = options;
+  } else if (options !== undefined && typeof options !== 'object') {
+    validateFunction(arguments.length === 2 ? options : callback, 'callback');
+    throw new ERR_INVALID_ARG_TYPE('options', ['integer', 'object'], options);
   } else {
-    validateCallback(callback);
+    validateFunction(callback, 'callback');
 
-    if (options !== null && typeof options === 'object') {
-      if (options.hints != null && typeof options.hints !== 'number') {
-        emitTypeCoercionDeprecationWarning();
-      }
+    if (options?.hints != null) {
+      validateNumber(options.hints, 'options.hints');
       hints = options.hints >>> 0;
-      if (options.family != null && typeof options.family !== 'number') {
-        emitTypeCoercionDeprecationWarning();
-      }
-      family = options.family >>> 0;
-      if (options.all != null && typeof options.all !== 'boolean') {
-        emitTypeCoercionDeprecationWarning();
-      }
-      all = options.all === true;
-      if (typeof options.verbatim === 'boolean') {
-        verbatim = options.verbatim === true;
-      } else if (options.verbatim != null) {
-        emitTypeCoercionDeprecationWarning();
-      }
-
       validateHints(hints);
-    } else {
-      if (options != null && typeof options !== 'number') {
-        emitTypeCoercionDeprecationWarning();
-      }
-      family = options >>> 0;
+    }
+    if (options?.family != null) {
+      validateOneOf(options.family, 'options.family', validFamilies, true);
+      family = options.family;
+    }
+    if (options?.all != null) {
+      validateBoolean(options.all, 'options.all');
+      all = options.all;
+    }
+    if (options?.verbatim != null) {
+      validateBoolean(options.verbatim, 'options.verbatim');
+      verbatim = options.verbatim;
     }
   }
 
-  validateOneOf(family, 'family', [0, 4, 6]);
-
   if (!hostname) {
     emitInvalidHostnameWarning(hostname);
     if (all) {
@@ -220,7 +219,7 @@ function lookupService(address, port, callback) {
 
   validatePort(port);
 
-  validateCallback(callback);
+  validateFunction(callback, 'callback');
 
   port = +port;
 
@@ -269,7 +268,7 @@ function resolver(bindingName) {
     }
 
     validateString(name, 'name');
-    validateCallback(callback);
+    validateFunction(callback, 'callback');
 
     const req = new QueryReqWrap();
     req.bindingName = bindingName;
