@@ -16,7 +16,6 @@ const {
   ERR_INSPECTOR_NOT_CONNECTED,
   ERR_INSPECTOR_NOT_ACTIVE,
   ERR_INSPECTOR_NOT_WORKER,
-  ERR_INVALID_ARG_TYPE,
 } = require('internal/errors').codes;
 
 const { hasInspector } = internalBinding('config');
@@ -26,7 +25,8 @@ if (!hasInspector)
 const EventEmitter = require('events');
 const { queueMicrotask } = require('internal/process/task_queues');
 const {
-  validateCallback,
+  validateFunction,
+  validateObject,
   validateString,
 } = require('internal/validators');
 const { isMainThread } = require('worker_threads');
@@ -53,6 +53,10 @@ class Session extends EventEmitter {
     this[messageCallbacksSymbol] = new SafeMap();
   }
 
+  /**
+   * Connects the session to the inspector back-end.
+   * @returns {void}
+   */
   connect() {
     if (this[connectionSymbol])
       throw new ERR_INSPECTOR_ALREADY_CONNECTED('The inspector session');
@@ -60,6 +64,11 @@ class Session extends EventEmitter {
       new Connection((message) => this[onMessageSymbol](message));
   }
 
+  /**
+   * Connects the session to the main thread
+   * inspector back-end.
+   * @returns {void}
+   */
   connectToMainThread() {
     if (isMainThread)
       throw new ERR_INSPECTOR_NOT_WORKER();
@@ -93,17 +102,24 @@ class Session extends EventEmitter {
     }
   }
 
+  /**
+   * Posts a message to the inspector back-end.
+   * @param {string} method
+   * @param {Record<unknown, unknown>} [params]
+   * @param {Function} [callback]
+   * @returns {void}
+   */
   post(method, params, callback) {
     validateString(method, 'method');
     if (!callback && typeof params === 'function') {
       callback = params;
       params = null;
     }
-    if (params && typeof params !== 'object') {
-      throw new ERR_INVALID_ARG_TYPE('params', 'Object', params);
+    if (params) {
+      validateObject(params, 'params');
     }
     if (callback) {
-      validateCallback(callback);
+      validateFunction(callback, 'callback');
     }
 
     if (!this[connectionSymbol]) {
@@ -120,6 +136,12 @@ class Session extends EventEmitter {
     this[connectionSymbol].dispatch(JSONStringify(message));
   }
 
+  /**
+   * Immediately closes the session, all pending
+   * message callbacks will be called with an
+   * error.
+   * @returns {void}
+   */
   disconnect() {
     if (!this[connectionSymbol])
       return;
@@ -134,6 +156,13 @@ class Session extends EventEmitter {
   }
 }
 
+/**
+ * Activates inspector on host and port.
+ * @param {number} [port]
+ * @param {string} [host]
+ * @param {boolean} [wait]
+ * @returns {void}
+ */
 function inspectorOpen(port, host, wait) {
   if (isEnabled()) {
     throw new ERR_INSPECTOR_ALREADY_ACTIVATED();
@@ -143,6 +172,12 @@ function inspectorOpen(port, host, wait) {
     waitForDebugger();
 }
 
+/**
+ * Blocks until a client (existing or connected later)
+ * has sent the `Runtime.runIfWaitingForDebugger`
+ * command.
+ * @returns {void}
+ */
 function inspectorWaitForDebugger() {
   if (!waitForDebugger())
     throw new ERR_INSPECTOR_NOT_ACTIVE();
@@ -151,7 +186,7 @@ function inspectorWaitForDebugger() {
 module.exports = {
   open: inspectorOpen,
   close: process._debugEnd,
-  url: url,
+  url,
   waitForDebugger: inspectorWaitForDebugger,
   // This is dynamically added during bootstrap,
   // where the console from the VM is still available
