@@ -271,7 +271,7 @@ const validateByteSource = hideStackFrames((val, name) => {
   val = toBuf(val);
 
   if (isAnyArrayBuffer(val) || isArrayBufferView(val))
-    return;
+    return val;
 
   throw new ERR_INVALID_ARG_TYPE(
     name,
