@@ -6,8 +6,9 @@ const {
   ObjectPrototypeHasOwnProperty,
   PromisePrototypeThen,
   PromiseResolve,
+  StringPrototypeSlice,
 } = primordials;
-const { extname } = require('path');
+const { basename, extname, relative } = require('path');
 const { getOptionValue } = require('internal/options');
 const { fetchModule } = require('internal/modules/esm/fetch_module');
 const {
@@ -20,7 +21,7 @@ const experimentalNetworkImports =
   getOptionValue('--experimental-network-imports');
 const experimentalSpecifierResolution =
   getOptionValue('--experimental-specifier-resolution');
-const { getPackageType } = require('internal/modules/esm/resolve');
+const { getPackageType, getPackageScopeConfig } = require('internal/modules/esm/resolve');
 const { URL, fileURLToPath } = require('internal/url');
 const { ERR_UNKNOWN_FILE_EXTENSION } = require('internal/errors').codes;
 
@@ -32,6 +33,10 @@ const protocolHandlers = ObjectAssign(ObjectCreate(null), {
   'node:'() { return 'builtin'; },
 });
 
+/**
+ * @param {URL} parsed
+ * @returns {string | null}
+ */
 function getDataProtocolModuleFormat(parsed) {
   const { 1: mime } = RegExpPrototypeExec(
     /^([^/]+\/[^;,]+)(?:[^,]*?)(;base64)?,/,
@@ -41,8 +46,15 @@ function getDataProtocolModuleFormat(parsed) {
   return mimeToFormat(mime);
 }
 
+/**
+ * @param {URL} url
+ * @param {{parentURL: string}} context
+ * @param {boolean} ignoreErrors
+ * @returns {string}
+ */
 function getFileProtocolModuleFormat(url, context, ignoreErrors) {
-  const ext = extname(url.pathname);
+  const filepath = fileURLToPath(url);
+  const ext = extname(filepath);
   if (ext === '.js') {
     return getPackageType(url) === 'module' ? 'module' : 'commonjs';
   }
@@ -53,12 +65,29 @@ function getFileProtocolModuleFormat(url, context, ignoreErrors) {
   if (experimentalSpecifierResolution !== 'node') {
     // Explicit undefined return indicates load hook should rerun format check
     if (ignoreErrors) return undefined;
-    throw new ERR_UNKNOWN_FILE_EXTENSION(ext, fileURLToPath(url));
+    let suggestion = '';
+    if (getPackageType(url) === 'module' && ext === '') {
+      const config = getPackageScopeConfig(url);
+      const fileBasename = basename(filepath);
+      const relativePath = StringPrototypeSlice(relative(config.pjsonPath, filepath), 1);
+      suggestion = 'Loading extensionless files is not supported inside of ' +
+        '"type":"module" package.json contexts. The package.json file ' +
+        `${config.pjsonPath} caused this "type":"module" context. Try ` +
+        `changing ${filepath} to have a file extension. Note the "bin" ` +
+        'field of package.json can point to a file with an extension, for example ' +
+        `{"type":"module","bin":{"${fileBasename}":"${relativePath}.js"}}`;
+    }
+    throw new ERR_UNKNOWN_FILE_EXTENSION(ext, filepath, suggestion);
   }
 
   return getLegacyExtensionFormat(ext) ?? null;
 }
 
+/**
+ * @param {URL} url
+ * @param {{parentURL: string}} context
+ * @returns {Promise<string> | undefined} only works when enabled
+ */
 function getHttpProtocolModuleFormat(url, context) {
   if (experimentalNetworkImports) {
     return PromisePrototypeThen(
@@ -70,6 +99,11 @@ function getHttpProtocolModuleFormat(url, context) {
   }
 }
 
+/**
+ * @param {URL | URL['href']} url
+ * @param {{parentURL: string}} context
+ * @returns {Promise<string> | string | undefined} only works when enabled
+ */
 function defaultGetFormatWithoutErrors(url, context) {
   const parsed = new URL(url);
   if (!ObjectPrototypeHasOwnProperty(protocolHandlers, parsed.protocol))
@@ -77,6 +111,11 @@ function defaultGetFormatWithoutErrors(url, context) {
   return protocolHandlers[parsed.protocol](parsed, context, true);
 }
 
+/**
+ * @param {URL | URL['href']} url
+ * @param {{parentURL: string}} context
+ * @returns {Promise<string> | string | undefined} only works when enabled
+ */
 function defaultGetFormat(url, context) {
   const parsed = new URL(url);
   return ObjectPrototypeHasOwnProperty(protocolHandlers, parsed.protocol) ?
