@@ -110,21 +110,21 @@ int SSL_CTX_use_certificate_chain(SSL_CTX* ctx,
   // Try getting issuer from a cert store
   if (ret) {
     if (issuer == nullptr) {
-      // TODO(tniessen): SSL_CTX_get_issuer does not allow the caller to
-      // distinguish between a failed operation and an empty result. Fix that
-      // and then handle the potential error properly here (set ret to 0).
-      *issuer_ = SSL_CTX_get_issuer(ctx, x.get());
+      ret = SSL_CTX_get_issuer(ctx, x.get(), &issuer);
+      ret = ret < 0 ? 0 : 1;
       // NOTE: get_cert_store doesn't increment reference count,
       // no need to free `store`
     } else {
       // Increment issuer reference count
-      issuer_->reset(X509_dup(issuer));
-      if (!*issuer_) {
+      issuer = X509_dup(issuer);
+      if (issuer == nullptr) {
         ret = 0;
       }
     }
   }
 
+  issuer_->reset(issuer);
+
   if (ret && x != nullptr) {
     cert->reset(X509_dup(x.get()));
     if (!*cert)
@@ -283,6 +283,7 @@ Local<FunctionTemplate> SecureContext::GetConstructorTemplate(
     env->SetProtoMethod(tmpl, "close", Close);
     env->SetProtoMethod(tmpl, "loadPKCS12", LoadPKCS12);
     env->SetProtoMethod(tmpl, "setTicketKeys", SetTicketKeys);
+    env->SetProtoMethod(tmpl, "setFreeListLength", SetFreeListLength);
     env->SetProtoMethod(tmpl, "enableTicketKeyCallback",
         EnableTicketKeyCallback);
 
@@ -362,6 +363,7 @@ void SecureContext::RegisterExternalReferences(
   registry->Register(Close);
   registry->Register(LoadPKCS12);
   registry->Register(SetTicketKeys);
+  registry->Register(SetFreeListLength);
   registry->Register(EnableTicketKeyCallback);
   registry->Register(GetTicketKeys);
   registry->Register(GetCertificate<true>);
@@ -499,8 +501,8 @@ void SecureContext::Init(const FunctionCallbackInfo<Value>& args) {
       max_version = TLS1_2_VERSION;
       method = TLS_client_method();
     } else {
-      THROW_ERR_TLS_INVALID_PROTOCOL_METHOD(
-          env, "Unknown method: %s", *sslmethod);
+      const std::string msg("Unknown method: ");
+      THROW_ERR_TLS_INVALID_PROTOCOL_METHOD(env, (msg + * sslmethod).c_str());
       return;
     }
   }
@@ -539,9 +541,9 @@ void SecureContext::Init(const FunctionCallbackInfo<Value>& args) {
   // OpenSSL 1.1.0 changed the ticket key size, but the OpenSSL 1.0.x size was
   // exposed in the public API. To retain compatibility, install a callback
   // which restores the old algorithm.
-  if (CSPRNG(sc->ticket_key_name_, sizeof(sc->ticket_key_name_)).is_err() ||
-      CSPRNG(sc->ticket_key_hmac_, sizeof(sc->ticket_key_hmac_)).is_err() ||
-      CSPRNG(sc->ticket_key_aes_, sizeof(sc->ticket_key_aes_)).is_err()) {
+  if (RAND_bytes(sc->ticket_key_name_, sizeof(sc->ticket_key_name_)) <= 0 ||
+      RAND_bytes(sc->ticket_key_hmac_, sizeof(sc->ticket_key_hmac_)) <= 0 ||
+      RAND_bytes(sc->ticket_key_aes_, sizeof(sc->ticket_key_aes_)) <= 0) {
     return THROW_ERR_CRYPTO_OPERATION_FAILED(
         env, "Error generating ticket keys");
   }
@@ -1117,6 +1119,9 @@ void SecureContext::SetTicketKeys(const FunctionCallbackInfo<Value>& args) {
 #endif  // !def(OPENSSL_NO_TLSEXT) && def(SSL_CTX_get_tlsext_ticket_keys)
 }
 
+void SecureContext::SetFreeListLength(const FunctionCallbackInfo<Value>& args) {
+}
+
 // Currently, EnableTicketKeyCallback and TicketKeyCallback are only present for
 // the regression test in test/parallel/test-https-resume-after-renew.js.
 void SecureContext::EnableTicketKeyCallback(
@@ -1239,14 +1244,11 @@ int SecureContext::TicketCompatibilityCallback(SSL* ssl,
 
   if (enc) {
     memcpy(name, sc->ticket_key_name_, sizeof(sc->ticket_key_name_));
-    if (CSPRNG(iv, 16).is_err() ||
-        EVP_EncryptInit_ex(
-            ectx, EVP_aes_128_cbc(), nullptr, sc->ticket_key_aes_, iv) <= 0 ||
-        HMAC_Init_ex(hctx,
-                     sc->ticket_key_hmac_,
-                     sizeof(sc->ticket_key_hmac_),
-                     EVP_sha256(),
-                     nullptr) <= 0) {
+    if (RAND_bytes(iv, 16) <= 0 ||
+        EVP_EncryptInit_ex(ectx, EVP_aes_128_cbc(), nullptr,
+                           sc->ticket_key_aes_, iv) <= 0 ||
+        HMAC_Init_ex(hctx, sc->ticket_key_hmac_, sizeof(sc->ticket_key_hmac_),
+                     EVP_sha256(), nullptr) <= 0) {
       return -1;
     }
     return 1;
