@@ -161,10 +161,10 @@ if (process.argv[2] === 'child') {
   child.stdout.on('data', (chunk) => { stdout += chunk; });
   child.on('exit', common.mustCall((code, signal) => {
     assert.strictEqual(stderr.trim(), '');
-    assert.deepStrictEqual(code, 0, 'Process exited unexpectedly with code: ' +
-                           `${code}`);
-    assert.deepStrictEqual(signal, null, 'Process should have exited cleanly,' +
-                            ` but did not: ${signal}`);
+    assert.strictEqual(code, 0, 'Process exited unexpectedly with code: ' +
+                       `${code}`);
+    assert.strictEqual(signal, null, 'Process should have exited cleanly,' +
+                       ` but did not: ${signal}`);
 
     const reports = helper.findReports(child.pid, tmpdir.path);
     assert.deepStrictEqual(reports, [], report_msg, reports);
